from typing import Callable, Union, Optional

from pydantic import BaseModel, computed_field


class QuoteField(BaseModel):
    name: str
    fid_list: list
    source_data: Optional[dict] = None
    value_func: Callable

    @computed_field(title='值')
    def value(self):
        return self.value_func(self.source_data)

    @computed_field(title='源值')
    def source_value(self):
        return [self.source_data[tmp] for tmp in self.fid_list]


class QuoteItem:
    """行情项"""

    def __init__(self, fid_list: list = None):
        """
        初始化创建
        :param fid_list: 依赖字段列表
        """
        self._source = None
        self._source_fun: Callable[[Union[str, int]], Union[str, int]] = None
        self._txt = None
        self._txt_fun: Callable[[Union[str, int]], str] = None
        self._color = None
        self._color_fun: Callable[[Union[str, int]], str] = None
        self._html = None
        self._html_fun: Callable[[Union[str, int]], str] = None
        self._blink_html = None
        self._blink_html_fun: Callable[[Union[str, int]], str] = None
        self.fid_list = fid_list
        self.create_fun: Callable[[Union[str, int]], 'QuoteItem'] = None

    def run(self, data_input):
        """运行"""
        if self.create_fun:
            tmp_item = self.create_fun(data_input)
            tmp_item.fid_list = self.fid_list
            return tmp_item
        if self._source_fun:
            self._source = self._source_fun(data_input)
        if self._txt_fun:
            self._txt = self._txt_fun(data_input)
        if self._color_fun:
            self._color = self._color_fun(data_input)
        if self._html_fun:
            self._html = self._html_fun(data_input)
        if self._blink_html_fun:
            self._blink_html = self._blink_html_fun(data_input)
        return self

    @property
    def source(self):
        return self._source

    def set_source(self, source: Union[int, str, Callable]) -> 'QuoteItem':
        if isinstance(source, Callable):
            self._source_fun = source
        else:
            self._source = source
        return self

    @property
    def txt(self):
        return self._txt if self._txt else self._source

    def set_txt(self, txt: Union[int, str, Callable]) -> 'QuoteItem':
        if isinstance(txt, Callable):
            self._txt_fun = txt
        else:
            self._txt = txt
        return self

    @property
    def html(self):
        color_str = f' style="color:{self._color};"' if self._color else ''
        return self._html if self._html else f'<span{color_str}>{self._source}</span>'

    def set_html(self, html: Union[int, str, Callable]) -> 'QuoteItem':
        if isinstance(html, Callable):
            self._html_fun = html
        else:
            self._html = html
        return self

    @property
    def color(self):
        return self._color if self._color else None

    def set_color(self, color: Union[int, str, Callable]) -> 'QuoteItem':
        if isinstance(color, Callable):
            self._color_fun = color
        else:
            self._color = color
        return self

    @property
    def blink_html(self):
        return self._blink_html if self._blink_html else f'<span>{self._source}</span>'

    def set_blink_html(self, blink_html: Union[int, str, Callable]) -> 'QuoteItem':
        if isinstance(blink_html, Callable):
            self._blink_html_fun = blink_html
        else:
            self._blink_html = blink_html
        return self

    def set_create_fun(self, create_fun: Callable[[Union[str, int]], 'QuoteItem']) -> 'QuoteItem':
        self.create_fun = create_fun
        return self



from typing import Optional

from pydantic import BaseModel, Field


class ArticleInfoItem(BaseModel):
    """文章信息项"""
    type_code: Optional[str] = Field(title='类型编码', default=None)
    code: str = Field(title='编码')
    title: str = Field(title='标题')
    sub_title: Optional[str] = Field(title='副标题', default=None)
    description: Optional[str] = Field(title='简介', default=None)
    publish_time: Optional[str] = Field(title='发布时间', default=None)


class ArticleDetailInfoItem(ArticleInfoItem):
    """文章详情信息项"""
    content: Optional[str] = Field(title='内容', default=None)



import re
from typing import Callable

from sqlalchemy import Column, DateTime, Boolean, Float, func, BigInteger
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker, Session
from sqlalchemy.engine import Connection

from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.ext.asyncio import async_sessionmaker

from afeng_tools.sqlalchemy_tools.core.sqlalchemy_items import DatabaseInfoItem
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_meta_class import ModelMetaClass


class SqlalchemyDb:
    """数据库对象"""

    def __init__(self, database_info: DatabaseInfoItem):
        self.database_info = database_info
        self.Base = declarative_base()
        self.BaseModel = self._get_base_model().BaseModel
        self.is_postgresql = True if re.match('^postgresql', self.database_info.database_uri) else False
        self.Base.is_postgresql = self.is_postgresql
        self.engine = create_engine(self.database_info.database_uri,
                                    echo=self.database_info.echo_sql,
                                    poolclass=self.database_info.pool_class,
                                    echo_pool=self.database_info.echo_pool,
                                    pool_size=self.database_info.pool_size,
                                    max_overflow=self.database_info.pool_max_overflow,
                                    pool_recycle=self.database_info.pool_recycle,
                                    pool_timeout=self.database_info.pool_timeout,
                                    pool_pre_ping=self.database_info.pool_pre_ping,
                                    pool_use_lifo=self.database_info.pool_use_lifo, )
        self.session_maker = sessionmaker(bind=self.engine,
                                          autocommit=self.database_info.auto_commit,
                                          autoflush=self.database_info.auto_flush,
                                          expire_on_commit=self.database_info.expire_on_commit)
        self.async_session_maker = async_sessionmaker(sync_session_class=self.session_maker)
        self.mapper_dict = {tmp_mapper.local_table.name: tmp_mapper.entity
                            for tmp_mapper in self.Base.registry.mappers}

    def _get_base_model(self):
        class _Model:
            class BaseModel(self.Base, metaclass=ModelMetaClass):
                """模型根类"""
                __abstract__ = True

                id = Column(BigInteger, comment='主键', primary_key=True, index=True, autoincrement=True)
                add_time = Column(DateTime, comment='添加时间', default=func.now())
                update_time = Column(DateTime, comment='修改时间', default=func.now(), onupdate=func.now())
                is_enable = Column(Boolean, comment='是否可用', default=True)
                order_num = Column(Float, comment='排序值', default=100)

        return _Model

    def get_session(self) -> Session:
        """获取Session"""
        return self.session_maker()

    def get_async_session(self) -> AsyncSession:
        """获取异步Session， 使用示例：await async_session.commit()"""
        return self.async_session_maker()

    def get_connection(self) -> Connection:
        """获取Connection, 使用指南：https://docs.sqlalchemy.org/en/20/core/connections.html#sqlalchemy.engine.Connection"""
        return self.engine.connect()

    def with_connection(self, work_func: Callable[[Connection, ], None]) -> Session:
        """使用Connection, 使用指南：https://docs.sqlalchemy.org/en/20/core/connections.html#sqlalchemy.engine.Connection"""
        with self.engine.connect() as connection:
            work_func(connection)



"""
sqlalchemy 的模型工具
"""
import json
from typing import Type

from afeng_tools.json_tool.json_tools import JsonEncoder


def to_dict(model):
    """Model实例转dict"""
    model_dict = dict(model.__dict__)
    del model_dict['_sa_instance_state']
    return model_dict


def to_dict2(model):
    """单个对象转dict(效果等同上面的那个)"""
    return {c.name: getattr(model, c.name) for c in model.__table__.columns}


def to_json(model) -> str:
    """model或model集合转换为json字符串"""
    if isinstance(model, list):
        return json.dumps([to_dict2(tmp) for tmp in model], cls=JsonEncoder, indent=4, ensure_ascii=False)
    else:
        return json.dumps(to_dict(model), cls=JsonEncoder, indent=4, ensure_ascii=False)


def list_to_dictlist(model_list):
    """多个对象转dict"""
    return [to_dict2(tmp) for tmp in model_list]


def list_to_json(model_list):
    """多个对象转json字符串"""
    return json.dumps([to_dict2(tmp) for tmp in model_list], cls=JsonEncoder, indent=4, ensure_ascii=False)


def copy_model(model):
    """复制模型，示例如下：
        info = GroupInfo(name='test')
        info_bak = model_utils.copy_model(info)
    """
    return type(model)(**to_dict(model))


def copy_model_data(old_model, new_model_class: Type):
    """
    复制Model数据
    :param old_model: 源model对象
    :param new_model_class: 目标Model类
    :return: 目标model对象
    """
    old_model_dict = to_dict(old_model)
    params_set = {c.name for c in new_model_class.__table__.columns}
    new_model_dict = {tmp_key: tmp_value for tmp_key, tmp_value in old_model_dict.items() if tmp_key in params_set}
    return new_model_class(**new_model_dict)



import os.path
from abc import ABCMeta, abstractmethod

from starlette.requests import Request

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.core.fastapi_response import json_resp
from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response
from afeng_tools.fastapi_tool.template.item import Error501DataItem, Error500DataItem, Error404DataItem, \
    TemplateError404AreaData, TemplateError501AreaData, TemplateError500AreaData
from afeng_tools.fastapi_tool.template.service import template_render_service
from afeng_tools.http_tool import http_request_tools


class ErrorService(metaclass=ABCMeta):

    @abstractmethod
    def handle_404(self, error_data: Error404DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return json_resp(error_no=404, message=error_data.message, sub_message=error_data.sub_message)

    @abstractmethod
    def handle_500(self, error_data: Error500DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return json_resp(error_no=500, message=error_data.message, sub_message=error_data.sub_message)

    @abstractmethod
    def handle_501(self, error_data: Error501DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return json_resp(error_no=501, message=error_data.message, sub_message=error_data.sub_message)


class DefaultErrorService(ErrorService):
    """默认错误处理"""

    def __init__(self, app_code: str = 'common'):
        self.app_code = app_code
        self.template_root_path = settings_tools.get_config(SettingsKeyEnum.server_template_path)

    def handle_response(self, request: Request,
                        error_no: int,
                        error_title: str,
                        error_data: Error404DataItem | Error500DataItem | Error501DataItem,
                        error_template_file: str = 'views/error/index.html',
                        context_data: dict = None,
                        app_info: AppInfo = None):
        if request and not http_request_tools.is_json(request.headers) and not (app_info and app_info.is_json_api):
            app_info = app_info if app_info else request.scope.get('app_info')
            if context_data and 'template_file' in context_data:
                error_index_template_file = context_data['template_file']
            else:
                error_index_template_file = f'{self.app_code}/{error_template_file}'
                if app_info:
                    if self.template_root_path:
                        if os.path.exists(
                                os.path.join(str(self.template_root_path), f'{app_info.code}/{error_template_file}')):
                            error_index_template_file = f'{app_info.code}/{error_template_file}'
            if os.path.exists(
                    os.path.join(str(self.template_root_path), error_index_template_file)):
                if context_data is None:
                    context_data = dict()
                if app_info and 'app_info' not in context_data:
                    context_data['app_info'] = app_info
                if 'is_mobile' not in context_data:
                    context_data['is_mobile'] = http_request_tools.is_mobile(request.headers.get('user-agent'))
                if 'title' not in context_data:
                    context_data['title'] = error_title
                if 'message' not in context_data:
                    context_data['message'] = error_data.message
                if 'sub_message' not in context_data:
                    context_data['sub_message'] = error_data.sub_message

                return create_template_response(request=request,
                                                template_file=error_index_template_file,
                                                context=context_data)
        return json_resp(error_no=error_no, message=error_data.message, sub_message=error_data.sub_message)

    def handle_404(self, error_data: Error404DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        if context_data is None:
            context_data = dict()
        is_mobile = http_request_tools.is_mobile(request.headers.get('user-agent'))
        if 'is_mobile' not in context_data:
            context_data['is_mobile'] = is_mobile
        if 'template_error_area' not in context_data:
            _error_area, _error_file = template_render_service.get_template_error_404_area(
                group_code='error/404',
                context_data_func=lambda: TemplateError404AreaData(is_mobile=is_mobile, **error_data.model_dump()),
                app_code=self.app_code,
                reload_freq=ReloadFreqEnum.always
            )
            context_data['template_error_area'] = _error_file
        return self.handle_response(request=request,
                                    error_no=404,
                                    error_title='404-Not Found',
                                    error_data=error_data,
                                    context_data=context_data,
                                    app_info=app_info)

    def handle_500(self, error_data: Error500DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        if context_data is None:
            context_data = dict()
        is_mobile = http_request_tools.is_mobile(request.headers.get('user-agent'))
        if 'is_mobile' not in context_data:
            context_data['is_mobile'] = is_mobile
        if 'template_error_area' not in context_data:
            _error_area, _error_file = template_render_service.get_template_error_500_area(
                group_code='error/500',
                context_data_func=lambda: TemplateError500AreaData(is_mobile=is_mobile, **error_data.model_dump()),
                app_code=self.app_code,
                reload_freq=ReloadFreqEnum.always
            )
            context_data['template_error_area'] = _error_file
        return self.handle_response(request=request,
                                    error_no=500,
                                    error_title='500-服务器错误',
                                    error_data=error_data,
                                    context_data=context_data,
                                    app_info=app_info)

    def handle_501(self, error_data: Error501DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        if context_data is None:
            context_data = dict()
        is_mobile = http_request_tools.is_mobile(request.headers.get('user-agent'))
        if 'is_mobile' not in context_data:
            context_data['is_mobile'] = is_mobile
        if 'template_error_area' not in context_data:
            _error_area, _error_file = template_render_service.get_template_error_501_area(
                group_code='error/501',
                context_data_func=lambda: TemplateError501AreaData(is_mobile=is_mobile, **error_data.model_dump()),
                app_code=self.app_code,
                reload_freq=ReloadFreqEnum.always
            )
            context_data['template_error_area'] = _error_file
        return self.handle_response(request=request,
                                    error_no=501,
                                    error_title='501-操作失败',
                                    error_data=error_data,
                                    context_data=context_data,
                                    app_info=app_info)




"""
列表工具
"""
import math
from typing import Any


def divide_to_groups(list_value: list[Any], group_length: int = 1000) -> list[list[Any]]:
    """
    将列表进行分组
    :param list_value: 列表值, 如：[1,2,3,4,5]
    :param group_length: 分组长度
    :return: 分组后的列表， 如：[[1,2],[3,4],[5]]
    """
    result_list = []
    group_count = math.ceil(len(list_value) / group_length)
    for i in range(group_count):
        result_list.append(list_value[i * group_length:(i + 1) * group_length])
    return result_list


def list_to_subgroup(group_by_attr: str, data_list: list) -> dict[Any, list]:
    """
    list列表拆分成子分组列表
    :param group_by_attr:分组字段
    :param data_list:数据列表
    :return: {分组字段值：[分组对应列表]}
    """
    result_dict = dict()
    for tmp_data in data_list:
        tmp_attr_value = getattr(tmp_data, group_by_attr)
        tmp_list = result_dict.get(tmp_attr_value)
        if tmp_list is None:
            result_dict[tmp_attr_value] = [tmp_data]
        else:
            tmp_list.append(tmp_data)
    return result_dict



import os
from typing import Callable

from aligo import Aligo, BaseShareFile, BaseFile

from afeng_tools.aliyun_pan_tool import aliyun_pan_tools, aliyun_pan_share_tools
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import UploadCreateResult
from afeng_tools.baidu_pan_tool.tools.baidu_pan_file_path_tools import format_pan_path
from afeng_tools.baidu_pan_tool.transfer_tool.core.alipan_transfer_to_baidupan import alipan_file_save
from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.file_tool import file_tools
from afeng_tools.log_tool import loguru_tools
from afeng_tools.os_tool import os_tools

logger = loguru_tools.get_logger()


def alipan_to_baidupan(baidupan_access_token: str, ali_pan_path: str = '/',
                       alipan_file_filter_func: Callable[[Aligo, str, BaseFile], bool] = lambda a, x, y: True,
                       baidupan_save_path_func: Callable[[str, str], str] = lambda x, y: f'{x}/{y}',
                       baidupan_save_after_handle_func: Callable[[BaseFile, str, UploadCreateResult], None] = None,
                       is_resource_drive: bool = False,
                       is_init_file_md5: bool = True):
    """
    阿里云盘转存到百度网盘
    :param baidupan_access_token:百度网盘access_token
    :param ali_pan_path: 阿里云盘文件路径
     :param alipan_file_filter_func: 阿里云盘文件过滤函数，只有满足条件的文件才会进行转存，参数：(阿里云盘的路径(包含文件名): alipan_path, 阿里云盘的文件: alipan_file)-> True或False
    :param baidupan_save_path_func: 百度网盘路径函数：(阿里云盘的路径(不包含文件名，根路径为/): alipan_path, 阿里云盘的文件名: alipan_file_name)-> 百度网盘的路径
      :param baidupan_save_after_handle_func:  百度网盘保存后的处理函数，参数：(阿里网盘文件, 百度网盘保存路径（包含文件名）,上传保存结果)
      :param is_resource_drive: 是否是资源盘
      :param is_init_file_md5: 是否初始化文件的md5
    :return:
    """
    transfer_list_file = os.path.join(os_tools.get_user_home(),
                                      f'.alipan_transfer_list-{hashlib_tools.calc_md5(ali_pan_path)}')
    exist_file_list = []
    if os.path.exists(transfer_list_file):
        exist_file_list = file_tools.read_file_lines(transfer_list_file)
    # 阿里云盘
    alipan_api = aliyun_pan_tools.get_ali_api()
    if is_resource_drive:
        alipan_api.default_drive_id = alipan_api.v2_user_get().resource_drive_id
    aliyun_pan_tools.list_all_file(alipan_api, ali_pan_path,
                                   callback_func=lambda pan_path, pan_file: alipan_file_save(
                                       baidupan_access_token=baidupan_access_token,
                                       alipan_api=alipan_api,
                                       alipan_path=pan_path,
                                       alipan_file=pan_file,
                                       transfer_list_file=transfer_list_file,
                                       exist_file_list=exist_file_list,
                                       alipan_file_filter_func=alipan_file_filter_func,
                                       baidupan_save_path_func=baidupan_save_path_func,
                                       baidupan_save_after_handle_func=baidupan_save_after_handle_func,
                                       is_init_file_md5=is_init_file_md5))

    if os.path.exists(transfer_list_file):
        # os.remove(transfer_list_file)
        pass


def alipan_share_transfer_to_baidupan(baidupan_access_token: str, alipan_share_msg: str,
                                      alipan_file_filter_func: Callable[[Aligo, str, BaseShareFile], bool] = lambda a,
                                                                                                                    x,
                                                                                                                    y: True,
                                      baidupan_save_path_func: Callable[[str, str], str] = lambda x, y: f'{x}/{y}',
                                      baidupan_save_after_handle_func: Callable[
                                          [BaseFile, str, UploadCreateResult], None] = None,
                                      is_init_file_md5: bool = True):
    return alipan_share_to_baidupan(baidupan_access_token=baidupan_access_token,
                                    alipan_share_msg=alipan_share_msg,
                                    alipan_file_filter_func=alipan_file_filter_func,
                                    baidupan_save_path_func=baidupan_save_path_func,
                                    baidupan_save_after_handle_func=baidupan_save_after_handle_func,
                                    is_init_file_md5=is_init_file_md5)


def alipan_share_to_baidupan(baidupan_access_token: str, alipan_share_msg: str,
                             alipan_file_filter_func: Callable[[Aligo, str, BaseShareFile], bool] = lambda a, x,
                                                                                                           y: True,
                             baidupan_save_path_func: Callable[[str, str], str] = lambda x, y: f'{x}/{y}',
                             baidupan_save_after_handle_func: Callable[
                                 [BaseFile, str, UploadCreateResult], None] = None,
                             is_init_file_md5: bool = True):
    """
    阿里云盘的分享转存道百度网盘
    :param baidupan_access_token:百度网盘access_token
    :param alipan_share_msg: 阿里云盘分享信息
     :param alipan_file_filter_func: 阿里云盘文件过滤函数，只有满足条件的文件才会进行转存，参数：(阿里云盘的路径(包含文件名): alipan_path, 阿里云盘的文件: alipan_file)-> True或False
    :param baidupan_save_path_func: 百度网盘路径函数：(阿里云盘的路径(不包含文件名，根路径为/): alipan_path, 阿里云盘的文件名: alipan_file_name)-> 百度网盘的路径
      :param baidupan_save_after_handle_func:  百度网盘保存后的处理函数，参数：(百度网盘保存路径（包含文件名）,上传保存结果)
      :param is_init_file_md5: 是否初始化文件的md5
    :return:
    """
    logger.info(alipan_share_msg)
    # 获取阿里云盘中要上传的文件
    alipan_api = aliyun_pan_tools.get_ali_api()
    share_info_result = aliyun_pan_share_tools.get_share_info_by_link(alipan_api, share_msg=alipan_share_msg)

    transfer_list_file = os.path.join(os_tools.get_user_home(),
                                      f'.alipan_transfer_list-{share_info_result.share_id}')
    exist_file_list = []
    if os.path.exists(transfer_list_file):
        exist_file_list = file_tools.read_file_lines(transfer_list_file)
    alipan_share_token = aliyun_pan_share_tools.get_share_token(alipan_api, share_info_result.share_id,
                                                                share_info_result.share_pwd)
    aliyun_pan_share_tools.list_share_all_file(alipan_api, alipan_share_token,
                                               callback_func=lambda _, pan_path, pan_file: alipan_file_save(
                                                   baidupan_access_token=baidupan_access_token,
                                                   alipan_api=alipan_api,
                                                   alipan_path=pan_path,
                                                   alipan_file=pan_file,
                                                   transfer_list_file=transfer_list_file,
                                                   exist_file_list=exist_file_list,
                                                   alipan_file_filter_func=alipan_file_filter_func,
                                                   baidupan_save_path_func=baidupan_save_path_func,
                                                   baidupan_save_after_handle_func=baidupan_save_after_handle_func,
                                                   is_init_file_md5=is_init_file_md5))

    if os.path.exists(transfer_list_file):
        # os.remove(transfer_list_file)
        pass


if __name__ == '__main__':
    alipan_share_transfer_to_baidupan(baidupan_access_token='',
                                      alipan_share_msg='https://www.alipan.com/s/oqCriKvJDig',
                                      alipan_file_filter_func=lambda a, x, y: True,
                                      baidupan_save_path_func=lambda x,
                                                                     y: f'/apps/阿里云转存{format_pan_path(x)}/{format_pan_path(y)}')



import tarfile


def run_tar(tar_file: str, input_file_list: list[tuple[str, str]]):
    """
    进行tar操作
    :param tar_file: 如：example.tar.gz
    :param input_file_list: 如：[('VPN虚拟专用网安全实践教程.pdf', 'output/191662_VPN虚拟专用网安全实践教程.pdf')]
    :return:
    """
    with tarfile.open(tar_file, 'w:gz') as tar:
        for tmp_name, tmp_file in input_file_list:
            # 使用add方法将目录添加到tar文件中，arcname参数指定了归档中的名称
            tar.add(tmp_file, arcname=tmp_name)


def run_un_tar(tar_file: str, output_dir: str):
    """
    解压tar文件到目录
    :param tar_file: 如：example.tar.gz
    :param output_dir: 解压到的目录, 如：output/example
    :return:
    """
    with tarfile.open(tar_file, 'r') as tar:
        tar.extractall(path=output_dir)


if __name__ == '__main__':
    run_tar('output/example.tar.gz',
            input_file_list=[('example.pdf', 'output/example.pdf')])
    run_un_tar('output/example.tar.gz', output_dir='output2/example')


from typing import Any, Optional
from pydantic import BaseModel, Field

from afeng_tools.pydantic_tool.model.common_models import LinkItem


class TemplateBaseData(BaseModel):
    """模板基础数据"""
    # 是否是移动端
    is_mobile: Optional[bool] = False
    # 附加的数据字典
    data_dict: Optional[dict[str, Any]] = None


class PageDataItem(BaseModel):
    """分页数据项"""
    # 当前页
    current_page: Optional[int] = 1
    # 分页大小
    page_size: Optional[int] = 10
    # 总数量
    total_count: Optional[int] = 0
    # 总页数
    total_page: Optional[int] = 0
    # 数据列表
    data_list: Optional[list[Any]] = None


class CalendarDataItem(BaseModel):
    """日历项"""
    date: Optional[str] = Field(title='日期', default=None)
    value: Optional[Any] = Field(title='数据', default=None)
    href: Optional[str] = Field(title='链接地址', default=None)


class AppinfoDataItem(BaseModel):
    # 应用标题
    title: Optional[str] = None
    # logo图片
    logo_image: Optional[str] = '/static/image/logo/logo.png'
    # 应用链接
    url: Optional[str] = '/'
    # 百度统计id
    baidu_tm_id: Optional[str] = None


class Error404DataItem(BaseModel):
    """错误404数据"""
    # 页面标题
    title: Optional[str] = '404-Not Found'
    # 错误信息
    message: Optional[str] = '很抱歉，找不到网页！'
    # 子消息
    sub_message: Optional[str] = '您访问的页面不存在或已被删除！ (｡•ˇ‸ˇ•｡)'


class Error500DataItem(BaseModel):
    """错误500数据"""
    # 页面标题
    title: Optional[str] = '500-服务器错误'
    # 错误信息
    message: Optional[str] = '服务器内部错误（Internal Server Error）'
    # 子消息
    sub_message: Optional[str] = '当您看到这个页面，表示服务器内部错误，此网站可能遇到技术问题，无法执行您的请求，请稍后重试或联系管理员进行处理，感谢您的支持！'
    # 等待秒数
    wait_seconds: Optional[int] = 30
    # 管理员联系方式， 如：mailto:afengbook@aliyun.com
    contact_info_url: Optional[str] = None


class Error501DataItem(BaseModel):
    """错误501数据"""
    # 页面标题
    title: Optional[str] = '501-操作失败'
    # 错误信息，如：很抱歉，无法完成您的操作
    message: Optional[str] = '操作失败'
    # 子消息， 如：很抱歉，无法完成您的操作，请进行<a href="https://txc.qq.com/products/622359" target="_blank">问题反馈</a>，感谢您的支持！
    sub_message: Optional[str] = '如果要解决该问题，请进行问题反馈，感谢您的支持！'
    # 管理员联系方式， 如：mailto:afengbook@aliyun.com
    contact_info_url: Optional[str] = None
    # 问题反馈链接, 如：https://txc.qq.com/products/622359
    feedback_url: Optional[str] = None


class RankingAreaTemplateItem(TemplateBaseData):
    """排行榜区域模板项"""
    # 区域标题
    title: Optional[str] = None
    # 更多按钮
    more_btn: Optional[LinkItem] = None
    # 列表项
    item_list: Optional[list[LinkItem]] = None


class HotTagAreaTemplateItem(TemplateBaseData):
    """热点标签区域模板项"""
    # 区域标题
    title: Optional[str] = None
    # 更多按钮
    more_btn: Optional[LinkItem] = None
    # 列表项
    item_list: Optional[list[LinkItem]] = None


class RecentItem(LinkItem):
    """最近更新项"""
    # 类型
    type_item: Optional[LinkItem] = None
    time_item: Optional[LinkItem] = None


class RecentAreaTemplateItem(TemplateBaseData):
    """最近更新区域模板项"""
    # 区域标题
    title: Optional[str] = None
    # 更多按钮
    more_btn: Optional[LinkItem] = None
    # 列表项
    item_list: Optional[list[RecentItem]] = None



from typing import Literal

from aligo import Aligo, BaseDrive


def get_drive(alipan_api: Aligo,
              drive_name: Literal[
                  'resource', 'sharedAlbum', 'alibum', 'note_drive', 'Default'] = 'Default', ) -> BaseDrive:
    drive_list = alipan_api.list_my_drives()
    for tmp_driver in drive_list:
        if drive_name == 'Default':
            if tmp_driver.drive_id == alipan_api.get_user().default_drive_id:
                return tmp_driver
        elif tmp_driver.drive_name == drive_name:
            return tmp_driver


def get_resource_drive(alipan_api: Aligo) -> BaseDrive:
    """获取资源盘"""
    return get_drive(alipan_api, drive_name='resource')


def get_default_drive(alipan_api: Aligo) -> BaseDrive:
    """获取默认盘"""
    return get_drive(alipan_api, drive_name='Default')



"""
sqlalchemy设置
"""
# 数据库字典
__DATABASE_DICT_CACHE__ = dict()

from afeng_tools.sqlalchemy_tools.core.sqlalchemy_class import SqlalchemyDb
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_items import DatabaseInfoItem


def set_database(database_info: DatabaseInfoItem, db_code: str = 'default'):
    """
    设置数据库工具
    :param database_info: 数据库信息
    :param db_code: 数据库编码
    :return:
    """
    __DATABASE_DICT_CACHE__[db_code] = SqlalchemyDb(database_info)


def get_database(db_code: str = 'default') -> SqlalchemyDb:
    """
    获取数据库工具
    :param db_code: 数据库编码
    :return: SqlalchemyDbTool
    """
    db_tool = __DATABASE_DICT_CACHE__.get(db_code)
    if db_tool is None:
        raise RuntimeError(
            '数据库配置缺失！请使用sqlalchemy_settings.set_database(db_code="afeng_api_db'
            '", DatabaseInfoItem(database_uri="postgresql://www_user:123456@127.0.0.1:5432/afeng-api-db"))进行数据库配置！')
    return db_tool



"""
- pip install beautifulsoup4 -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
- pip install pyquery -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os.path

from bs4 import BeautifulSoup
from pyquery import PyQuery

from afeng_tools.crawler_tool import pyquery_tools
from afeng_tools.file_tool import file_tools
from afeng_tools.template_tool.core import js_tools, less_tools


def _render_html(html_dom: PyQuery, template_name: str) -> str:
    """渲染html代码"""
    template_type = html_dom("template").attr('type')
    template_value = html_dom("template").attr('value')
    if template_type == 'macro':
        html_dom('template>div:first-child').html(
            html_dom('template>div:first-child').outer_html(method="xml")
        ).attr('class', template_value).remove_attr('id')
    elif template_type == 'macro-child':
        pass
    template_html = html_dom("template").html()
    html_bs = BeautifulSoup(template_html, 'html.parser')
    if template_type == 'macro':
        return html_bs.prettify()
    else:
        return f'<div id="{template_name}">{html_bs.prettify()}</div>'


def _render_js(html_dom: PyQuery, template_name: str, parent_html_dom: PyQuery = None) -> str:
    """渲染js代码"""
    js_list = []
    for item in html_dom('script').items():
        if item.attr('src'):
            js_list.append(item.outer_html())
        else:
            tmp_js_list = list()
            tmp_js_list.append('<script>')
            tmp_js_list.append(js_tools.min_js('var ' + template_name + ' = ' +
                                               ''.join(item.contents()).strip().lstrip('export')
                                               .strip().lstrip('default').strip() + ';'))
            tmp_js_list.append(f'{template_name}.methods.data={template_name}.data;')
            tmp_js_list.append(f'{template_name}.methods.init();')
            tmp_js_list.append('</script>')
            js_list.append(''.join(tmp_js_list))
    parent_js = ''
    if parent_html_dom:
        parent_js = parent_html_dom('script').outer_html()
    return parent_js + ''.join(js_list)


def _render_css(html_dom: PyQuery, template_name: str, less_dir: str) -> str:
    """渲染css代码"""
    template_type = html_dom("template").attr('type')
    template_value = html_dom("template").attr('value')
    parent_style = ''
    style_list = list()
    if template_type == 'macro':
        style_list.append('.' + template_value + '{')
    else:
        style_list.append('#' + template_name + '{')
    for tmp in html_dom('style').contents():
        style_list.append(tmp)
    style_list.append('}')
    return parent_style + str(html_dom('link')) + \
        '<style>' + less_tools.compile_to_css(''.join(style_list), less_dir=less_dir) + '</style>'


def _get_parent_html_dom(html_dom: PyQuery, template_root_path: str):
    template_parent = html_dom("template").attr('parent')
    if template_parent:
        parent_template_file = os.path.join(template_root_path, template_parent)
        if os.path.exists(parent_template_file):
            return pyquery_tools.get_PyQuery_from_html(file_tools.read_file(parent_template_file))


def _render_macro_html(html_dom: PyQuery, html_code: str, js_code: str, css_code: str):
    if html_dom("template").attr('type') == 'macro':
        html_code = html_code.rstrip().rstrip('%}').rstrip().rstrip('endmacro') \
            .rstrip().rstrip('-').rstrip().rstrip('{%')
        html_code = html_code + js_code + css_code
        html_code = html_code + '{%- endmacro %}'
    return html_code


def render(template_file: str, template_root_path: str):
    """
    格式化模板
    :param template_file: 模板文件，类似vue模板
    :param template_root_path: 模板根路径
    :return:(html代码，js代码，css代码)
    """
    html_dom = pyquery_tools.get_PyQuery_from_html(file_tools.read_file(template_file))
    template_name = os.path.split(template_file)[1].rsplit('.', maxsplit=1)[0]
    template_name = template_name.replace('-', '_')
    template_name = 'template_' + template_name
    parent_html_dom = _get_parent_html_dom(html_dom, template_root_path=template_root_path)
    html_code = _render_html(html_dom, template_name)
    html_code = html_code.replace('&gt;', '>').replace('&lt;', '<')
    js_code = _render_js(html_dom, template_name, parent_html_dom)
    css_code = _render_css(html_dom, template_name, os.path.dirname(template_file))
    css_code = css_code.replace(' rem', 'rem')
    html_code = _render_macro_html(html_dom, html_code, js_code, css_code)
    return html_code, js_code, css_code



import os
from enum import Enum
from typing import TypeVar, Type, Callable
from fastapi.exceptions import RequestValidationError
from starlette.requests import Request

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.core.fastapi_exception_handler import HtmlFileResponseException
from afeng_tools.fastapi_tool.fastapi_request_tools import generate_request_id
from afeng_tools.fastapi_tool.template.fastapi_template_tools import is_need_reload
from afeng_tools.pydantic_tool.model.common_models import EnumItem


def convert_enum_type(param_name: str, enum_class: Type[Enum]) -> Callable:
    """
    转换枚举类型的装饰器， 使用示例：search_type: SearchTypeEnum = Depends(convert_enum_type('search_type', SearchTypeEnum)
    :param param_name: 参数名称
    :param enum_class: 枚举类型
    :return: 枚举
    """
    EnumType = TypeVar(enum_class.__name__, bound=enum_class)

    def convert_enum_type_wrapper(request: Request) -> EnumType:
        enum_type_str = request.query_params.get(param_name)
        if not enum_type_str:
            enum_type_str = request.path_params.get(param_name)
            if not enum_type_str and hasattr(request.form(), 'get'):
                enum_type_str = request.form().get(param_name)
        if enum_type_str:
            type_enum_list = [tmp for tmp in enum_class if tmp.name == enum_type_str or tmp.value == enum_type_str]
            if type_enum_list:
                return type_enum_list[0]
            else:
                type_enum_list = [tmp for tmp in enum_class if
                                  isinstance(tmp.value, EnumItem) and tmp.value.value == enum_type_str]
                if type_enum_list:
                    return type_enum_list[0]
        # raise RequestValidationError('请求参数[search_type]值有误！')

    return convert_enum_type_wrapper


def depend_cache_html_file(reload_freq: ReloadFreqEnum = ReloadFreqEnum.daily,
                           cache_path: str = None) -> Callable:
    """
    依赖（缓存html文件）
    :param reload_freq:
    :param cache_path: 缓存路径
    :return:
    """

    async def func_wrap(request: Request) -> str | None:
        if settings_tools.get_config('app.is_debug'):
            return None
        cache_dir = cache_path
        request_id = generate_request_id(request)
        if cache_dir is None:
            cache_dir = os.path.join(settings_tools.get_config('server.html_save_path'), 'cache')
        cache_html = os.path.join(cache_dir, f'{request_id}.html')
        if os.path.exists(cache_html) and not is_need_reload(cache_html, reload_freq=reload_freq):
            raise HtmlFileResponseException(html_file=cache_html, binary_flag=True)
        return cache_html

    return func_wrap



class AfengException(Exception):
    """自定义异常类，继承自内置的 Exception 类"""

    def __init__(self, message: str, sub_message: str = None):
        """初始化方法，可以接收一个消息作为参数"""
        self.message = message
        self.sub_message = sub_message

    def __str__(self):
        """返回异常的字符串表示，通常是异常的消息"""
        return f'{self.message}\n{self.sub_message}'


class HttpException(Exception):
    """自定义异常类，继承自内置的 Exception 类"""

    def __init__(self, status_code: int, message: str, sub_message: str = None):
        """初始化方法，可以接收一个消息作为参数"""
        self.status_code = status_code
        self.message = message
        self.sub_message = sub_message

    def __str__(self):
        """返回异常的字符串表示，通常是异常的消息"""
        return f'[{self.status_code}]{self.message}\n{self.sub_message}'



import datetime

import requests

from .. import aliyun_drive_auth_tools, aliyun_drive_share_tools, aliyun_drive_file_tools, \
    aliyun_drive_download_tools


def run():
    alipan_api = aliyun_drive_auth_tools.get_alipan_api(use_resource_drive=True)
    # child_list = aliyun_drive_file_tools.get_file_list(alipan_api, parent_file_id='66438c41fe482caa3a70492b86884b7e87532310')
    file_info = aliyun_drive_file_tools.get_file_by_path(alipan_api,
                                                         file_path='23红宝书时间规划表.pdf',
                                                         parent_file_id='664792aeefb2acd9e80b42ac9ad762339f0e2bbf')

    # download_url_resp = alipan_api.get_download_url(file_id=file_info.file_id)
    url = aliyun_drive_download_tools.get_download_url(alipan_api, file_id=file_info.file_id)
    resp = requests.get(url, headers={
        'Referer': 'https://www.aliyundrive.com/',
        'Origin': 'https://www.aliyundrive.com',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36'
    })
    print()


if __name__ == '__main__':
    run()
    result = datetime.datetime.fromtimestamp(1717443579)
    print()



"""
模块渲染服务
"""
import os
from typing import Callable

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.template import fastapi_template_tools
from afeng_tools.fastapi_tool.template.item import TemplateHtmlHeadData, TemplateBreadCrumbData, \
    TemplatePaginationAreaData, TemplateLeftNavData, \
    TemplatePageFooterData, TemplateResultListData, TemplateTagListData, \
    TemplateGroupListData, TemplateFilterTypeAreaData, TemplateDayCalendarData, TemplateTabPanelData, \
    TemplateTopRankingData, TemplateRedirectDownloadAreaData, TemplateRedirectAreaData, TemplateError404AreaData, \
    TemplateError500AreaData, TemplateError501AreaData, TemplateShowQrcodeAreaData, TemplateInfoAreaData, \
    TemplateAlertInfoAreaData, HotTagAreaTemplateItem, RecentAreaTemplateItem, RankingAreaTemplateItem
from afeng_tools.http_tool.http_url_tool import join_local_url
from afeng_tools.jinja2_tool import jinja2_tools
from afeng_tools.template_tool.template_decorator_tools import template_area

template_directory = settings_tools.get_config(SettingsKeyEnum.server_template_path)
template_env = jinja2_tools.create_template_env(directory_list=[template_directory])


def render(do_render_func: Callable, **kwargs) -> str:
    """渲染"""
    return do_render_func(**kwargs)


is_debug = settings_tools.get_config('app.is_debug')


def _render_template_file(app_code: str, template_file: str, group_code: str,
                          context_data_func: Callable,
                          reload_freq: ReloadFreqEnum = ReloadFreqEnum.weekly,
                          is_mobile: bool = True,
                          **kwargs):
    """渲染模板文件"""

    template_html_file = join_local_url([app_code, template_file])
    tmp_save_html_file = join_local_url(['tmp', app_code, group_code, 'mobile' if is_mobile else 'pc', template_file])
    save_template_file = str(os.path.join(template_directory, tmp_save_html_file.removeprefix('/')))
    if fastapi_template_tools.is_need_reload(save_template_file, reload_freq, is_debug=is_debug):
        if kwargs:
            context_data = context_data_func(**kwargs)
        else:
            context_data = context_data_func()
        template_content = jinja2_tools.format_template(template_html_file, context=context_data, env=template_env)
        os.makedirs(os.path.dirname(save_template_file), exist_ok=True)
        with open(save_template_file, 'w', encoding='utf-8') as f:
            f.write('{% raw %}' + template_content + '{% endraw %}')
    return tmp_save_html_file


@template_area(file='views/head/page_head.html')
def get_template_head(group_code: str, context_data_func: Callable[[], TemplateHtmlHeadData],
                      app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                      is_mobile: bool = True,
                      template_file: str = None) -> str:
    def context_data_func_wrapper():
        data = context_data_func()
        context_data = data.model_dump()
        context_data['head_title'] = data.title
        context_data['head_description'] = data.description
        context_data['head_keywords'] = ','.join([tmp for tmp in data.keyword_list if tmp])
        context_data['head_author'] = data.author
        context_data['head_favicon'] = data.favicon
        context_data['head_origin'] = data.origin
        return context_data

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='views/head/page_more_head.html')
def get_template_more_head(group_code: str, context_data_func: Callable[[], TemplateHtmlHeadData],
                           app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                           is_mobile: bool = True,
                           template_file: str = None) -> str:
    return get_template_head(group_code=group_code, context_data_func=context_data_func,
                             app_code=app_code, reload_freq=reload_freq, is_mobile=is_mobile,
                             template_file=template_file)


@template_area(file='module_html/page/bread_crumb.html')
def get_template_bread_crumb(group_code: str, context_data_func: Callable[[], TemplateBreadCrumbData],
                             app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                             is_mobile: bool = True,
                             template_file: str = None) -> str:
    """
    面包屑模板
    :param is_mobile:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code:
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return {
            'page_title': data.page_title,
            'data_list': data.bread_crumb_list,
        }

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/page/left_nav.html')
def get_template_left_nav(group_code: str, context_data_func: Callable[[], TemplateLeftNavData],
                          app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                          is_mobile: bool = True,
                          template_file: str = None) -> str:
    """
    左侧导航模板
    :param is_mobile:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code:
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/page_footer.html')
def get_template_page_footer(group_code: str, context_data_func: Callable[[], TemplatePageFooterData],
                             app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                             is_mobile: bool = True,
                             template_file: str = None) -> str:
    """
    页面底部模板
    :param is_mobile:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code:
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/area/ranking_area.html')
def get_template_ranking_area(group_code: str, context_data_func: Callable[[], RankingAreaTemplateItem],
                              reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                              is_mobile: bool = True,
                              app_code='common', template_file: str = None) -> str:
    """
    排行榜区域模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param is_mobile: 是否是移动端
    :param app_code: 应用编码
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/area/recent_area.html')
def get_template_recent_area(group_code: str, context_data_func: Callable[[], RecentAreaTemplateItem],
                             reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                             is_mobile: bool = True,
                             app_code='common', template_file: str = None) -> str:
    """
    最近更新区域模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param is_mobile: 是否是移动端
    :param app_code: 应用编码
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/area/hot_tag_area.html')
def get_template_hot_tag_area(group_code: str, context_data_func: Callable[[], HotTagAreaTemplateItem],
                              reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                              is_mobile: bool = True,
                              app_code='common', template_file: str = None) -> str:
    """
    热点标签区域模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param is_mobile: 是否是移动端
    :param app_code: 应用编码
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)

@template_area(file='module_html/list/result_list.html')
def get_template_result_list(app_code: str, group_code: str, context_data_func: Callable[[], TemplateResultListData],
                             reload_freq: ReloadFreqEnum = ReloadFreqEnum.always, is_mobile: bool = True,
                             template_file: str = None) -> str:
    """
    页面底部模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code:
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/list/pagination_area.html')
def get_template_pagination_area(group_code: str, context_data_func: Callable[[], TemplatePaginationAreaData],
                                 app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                 is_mobile: bool = True,
                                 template_file: str = None) -> str:
    """结果模板"""

    def context_data_func_wrapper():
        data = context_data_func()
        if data.pre_btn and data.pre_btn.title is None:
            data.pre_btn.title = '上一页'
        if data.next_btn and data.next_btn.title is None:
            data.next_btn.title = '下一页'
        return data.model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/list/tag_list.html')
def get_template_tag_list(group_code: str, context_data_func: Callable[[], TemplateTagListData],
                          app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                          is_mobile: bool = True,
                          template_file: str = None) -> str:
    """
    标签列表模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code:
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/list/group_list.html')
def get_template_group_list(group_code: str, context_data_func: Callable[[], TemplateGroupListData],
                            app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                            is_mobile: bool = True,
                            template_file: str = None) -> str:
    """
    分组列表模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code:
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/list/filter_type_area.html')
def get_template_filter_type_area(group_code: str, context_data_func: Callable[[], TemplateFilterTypeAreaData],
                                  app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                  is_mobile: bool = True,
                                  template_file: str = None) -> str:
    """过滤类型模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/list/day_calendar.html')
def get_template_day_calendar(group_code: str, context_data_func: Callable[[], TemplateDayCalendarData],
                              app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                              is_mobile: bool = True,
                              template_file: str = None) -> str:
    """结果模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/page/tab_panel.html')
def get_template_tab_panel(group_code: str, context_data_func: Callable[[], TemplateTabPanelData],
                           app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                           is_mobile: bool = True,
                           template_file: str = None) -> str:
    """Tab面板模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/ranking_list/top_ranking_list.html')
def get_template_top_ranking_list(group_code: str, context_data_func: Callable[[], TemplateTopRankingData],
                                  app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                  is_mobile: bool = True,
                                  template_file: str = None) -> str:
    """热点数据排行模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/redirect/download_area.html')
def get_template_redirect_download_area(group_code: str,
                                        context_data_func: Callable[[], TemplateRedirectDownloadAreaData],
                                        app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                        is_mobile: bool = True,
                                        template_file: str = None) -> str:
    """下载区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile, )


@template_area(file='module_html/redirect/redirect_area.html')
def get_template_redirect_area(group_code: str,
                               context_data_func: Callable[[], TemplateRedirectAreaData],
                               app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                               is_mobile: bool = True,
                               template_file: str = None) -> str:
    """下载区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/redirect/info_area.html')
def get_template_info_area(group_code: str,
                           context_data_func: Callable[[], TemplateInfoAreaData],
                           app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                           is_mobile: bool = True,
                           template_file: str = None) -> str:
    """信息区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/alert/show_qrcode_area.html')
def get_template_show_qrcode_area(group_code: str,
                                  context_data_func: Callable[[], TemplateShowQrcodeAreaData],
                                  app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                  is_mobile: bool = True,
                                  template_file: str = None) -> str:
    """显示二维码模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/alert/alert_info_area.html')
def get_template_alert_info_area(group_code: str,
                                 context_data_func: Callable[[], TemplateAlertInfoAreaData],
                                 app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                 is_mobile: bool = True,
                                 template_file: str = None) -> str:
    """警告信息区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/error/404_area.html')
def get_template_error_404_area(group_code: str,
                                context_data_func: Callable[[], TemplateError404AreaData],
                                app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                is_mobile: bool = True,
                                template_file: str = None) -> str:
    """404错误区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/error/500_area.html')
def get_template_error_500_area(group_code: str,
                                context_data_func: Callable[[], TemplateError500AreaData],
                                app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                is_mobile: bool = True,
                                template_file: str = None) -> str:
    """500错误区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/error/501_area.html')
def get_template_error_501_area(group_code: str,
                                context_data_func: Callable[[], TemplateError501AreaData],
                                app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                is_mobile: bool = True,
                                template_file: str = None) -> str:
    """500错误区域模板"""

    def context_data_func_wrapper():
        return context_data_func().model_dump()

    return _render_template_file(app_code, template_file,
                                 group_code=group_code, context_data_func=context_data_func_wrapper,
                                 reload_freq=reload_freq, is_mobile=is_mobile)



from typing import Callable

from scrapy import Selector
from scrapy.http import Response

from afeng_tools.scrapy_tool import scrapy_selector_tools
from afeng_tools.scrapy_tool.scrapy_selector_tools import select_first_value
from afeng_tools.scrapy_tool.scrapy_wiki.spider_wiki_items import WikiInfoRuleItem, WikiItemResult


def _handle_wiki_page_child_item(response: Response, selector: Selector, rule_item: WikiInfoRuleItem,
                                 item_result: WikiItemResult) -> list[WikiItemResult]:
    result_list = []
    child_item_el_list = scrapy_selector_tools.select(selector, rule_item.child_item_list_rule)
    if child_item_el_list:
        for child_item_index, child_item_el in enumerate(child_item_el_list):
            child_item_result_list = handle_wiki_page(response, selector=child_item_el,
                                                      rule_item=rule_item.child_item_info_rule,
                                                      parent_item_result=item_result)
            result_list.extend(child_item_result_list)
            if rule_item.is_recursion:
                result_list.extend(
                    _handle_wiki_page_child_item(response, child_item_el, rule_item, child_item_result_list[-1]))

    return result_list


def handle_wiki_page(response: Response, selector: Selector, rule_item: WikiInfoRuleItem,
                     parent_item_result: WikiItemResult = None) -> list[WikiItemResult]:
    """处理wiki的page页面"""
    result_list = []
    title = select_first_value(selector, rule_item.title_rule)
    url = select_first_value(selector, rule_item.url_rule)
    if parent_item_result:
        item_result = parent_item_result.model_copy(deep=True)
    else:
        item_result = WikiItemResult()
    item_result.title_list.append(title)
    item_result.source_url_list.append(url)
    if url:
        url = response.urljoin(url)
    item_result.url_list.append(url)
    item_result.content_rule = rule_item.page_content_rule
    result_list.append(item_result)
    if rule_item.child_item_list_rule and rule_item.child_item_info_rule:
        result_list.extend(_handle_wiki_page_child_item(response, selector, rule_item, item_result))
    return result_list


def has_value(data_list: list[WikiItemResult], new_value: WikiItemResult,
              equal_callback: Callable[[WikiItemResult, WikiItemResult], bool] = lambda x, y: '/'.join(
                  x.title_list) == '/'.join(y.title_list)) -> tuple[bool, int]:
    """
    判断列表中是否有item项
    :param data_list: item列表
    :param new_value: 需要判断是否存在的值
    :param equal_callback: 相等判断函数
    :return: (是否存在，位置索引)
    """
    for index, old_data in enumerate(data_list):
        if equal_callback(old_data, new_value):
            return True, index
    return False, -1


def get_insert_index(start_index: int, data_list: list[WikiItemResult],
                     equal_value_callback: Callable[[WikiItemResult], str] = lambda x: '/'.join(x.title_list)):
    """
    获取插入值的索引
    :param start_index: 开始索引
    :param data_list: 数据列表
    :param equal_value_callback: 判断相等的值回调
    :return:
    """
    tmp_title = equal_value_callback(data_list[start_index])
    tmp_data_list = data_list[start_index:]
    for index, tmp_data in enumerate(tmp_data_list):
        if not equal_value_callback(tmp_data).startswith(tmp_title):
            return start_index + index
    return start_index + len(tmp_data_list) + 1



import os
import re
import time
import typing
from abc import ABCMeta, abstractmethod, ABC

from starlette.middleware.base import _StreamingResponse
from starlette.requests import Request
from starlette.responses import Response, RedirectResponse, StreamingResponse

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.cache_tool import cache_memory_tools
from afeng_tools.fastapi_tool.fastapi_request_tools import generate_request_id
from afeng_tools.file_tool import tmp_file_tools
from afeng_tools.file_tool.file_tools import iterate_read_file


def request_set_no_cache(request: Request):
    request.scope['request_not_cache'] = True


class HttpCacher(metaclass=ABCMeta):
    def __init__(self, request: Request, app_info: AppInfo):
        self.request = request
        self.request_id = generate_request_id(request)
        self.app_info = app_info

    @abstractmethod
    def is_need_cache(self) -> bool:
        """是否需要缓存"""
        return False

    @abstractmethod
    def remove_cache(self):
        """移除缓存"""
        pass

    @abstractmethod
    def get_cache(self) -> tuple[bool, Response | None]:
        """
        获取缓存
        :return: （是否有缓存，缓存响应）
        """
        return False, None

    @abstractmethod
    async def save_cache(self, response: Response) -> Response | None:
        """
        保存缓存
        :param response: 响应内容
        :return: 如果返回None，则使用原response; 如果返回Response, 则响应Response
        """
        pass


class HttpRedirectCacher(HttpCacher, ABC):

    def is_need_cache(self) -> bool:
        """是否需要缓存"""
        return True

    def remove_cache(self):
        """移除缓存"""
        cache_memory_tools.delete_cache('redirect_cache', self.request_id)

    def get_cache(self) -> tuple[bool, Response | None]:
        cache_info = cache_memory_tools.get_time_cache('redirect_cache', self.request_id)
        if cache_info:
            cache_time, redirect_url = cache_info
            re_search = re.search('&(time|dstime)=(.*?)&', redirect_url)
            if re_search and time.time() >= (cache_time + 8 * 3600 - 100):
                cache_memory_tools.delete_cache('redirect_cache', self.request_id)
                return False, None
            return True, RedirectResponse(redirect_url)
        return False, None

    async def save_cache(self, response: Response):
        if response.status_code == 307:
            redirect_url = response.headers.get('location')
            re_search = re.search('&(time|dstime)=(.*?)&', redirect_url)
            if re_search:
                timestamp = float(re_search.group(2))
            else:
                timestamp = time.time()
            cache_memory_tools.add_time_cache('redirect_cache', self.request_id, redirect_url, timestamp=timestamp)


class HttpHtmlCacher(HttpCacher, ABC):

    def __init__(self, request: Request, app_info: AppInfo):
        super().__init__(request, app_info)
        self.request_cache_file = self._get_request_cache_file()

    def _get_request_cache_file(self) -> str:
        if self.app_info:
            save_child_path = self.request.url.path.removeprefix(f'{self.app_info.prefix}/')
            return str(os.path.join(settings_tools.get_config('server.html_save_path'),
                                    'cache', self.app_info.code, save_child_path, f'{self.request_id}.html'))
        else:
            save_child_path = self.request.url.path.removeprefix('/')
            return str(os.path.join(tmp_file_tools.get_user_tmp_dir(),
                                    'html_cache', save_child_path, f'{self.request_id}.html'))

    async def _request_save_cache(self, body_iterator: typing.AsyncIterable, charset: str, is_gzip=False):
        cache_file = self.request_cache_file
        if is_gzip:
            cache_file = cache_file + '.gzip'
        os.makedirs(os.path.dirname(cache_file), exist_ok=True)
        with open(cache_file, 'wb') as f:
            async for chunk in body_iterator:
                if not isinstance(chunk, bytes):
                    chunk = chunk.encode(charset)
                f.write(chunk)

    def is_need_cache(self) -> bool:
        """是否需要缓存"""
        return self.request.headers and self.request.headers.get('accept') and 'html' in self.request.headers.get(
            'accept')

    def remove_cache(self, is_gzip=False):
        """移除缓存"""
        cache_file = self.request_cache_file
        if is_gzip:
            cache_file = cache_file + '.gzip'
        if os.path.exists(cache_file):
            os.remove(cache_file)

    def get_cache(self) -> tuple[bool, Response | None]:
        """
        获取缓存
        :return: （是否有缓存，缓存响应）
        """
        if os.path.exists(self.request_cache_file):
            return True, StreamingResponse(content=iterate_read_file(self.request_cache_file, binary_flag=True),
                                           media_type="text/html")
        gzip_cache_html = self.request_cache_file + '.gzip'
        if os.path.exists(gzip_cache_html):
            return True, StreamingResponse(content=iterate_read_file(gzip_cache_html, binary_flag=True),
                                           headers={'content-length': str(os.stat(gzip_cache_html).st_size),
                                                    'content-type': 'text/html; charset=utf-8',
                                                    'content-encoding': 'gzip',
                                                    'vary': 'Accept-Encoding'})
        return False, None

    async def save_cache(self, response: Response):
        if response.status_code == 200 and response.headers.get('content-type') and response.headers.get(
                'content-type').startswith('text/html'):
            if isinstance(response, _StreamingResponse):
                await self._request_save_cache(response.body_iterator, response.charset,
                                               is_gzip=response.headers.get('content-encoding') == 'gzip')
                has_cache, resp = self.get_cache()
                return resp



from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class BaseService:
    po_service_type: PoService = None

    def __init__(self, db_code: str, po_model_type: type = None, app_code: str = None):
        self.app_code = app_code
        self.db_code = db_code
        if po_model_type is not None:
            self.po_model_type = po_model_type
        if self.po_service_type:
            if po_model_type is not None:
                self.po_service = self.po_service_type(db_code=self.db_code, model_type=self.po_model_type)
            else:
                self.po_service = self.po_service_type(db_code=self.db_code)
            if not hasattr(self, 'po_model_type') or self.po_model_type is None:
                self.po_model_type = self.po_service.model_type



"""
发送邮箱工具
"""
import email
import os
import smtplib
from email.mime.application import MIMEApplication
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr

from afeng_tools.http_tool import http_mimetype_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

log = get_logger()


def send_email(smtp_client: smtplib.SMTP_SSL, from_nickname: str, from_email: str, receiver_list: list[str],
               subject: str, email_content: str, is_html: bool = True, cc_list: list[str] = None,
               bcc_list: list[str] = None,
               image_attachments: dict[str, str] = None, file_attachments: dict[str, str] = None):
    if is_html:
        email_msg = MIMEText(email_content, 'html', 'utf-8')
    else:
        email_msg = MIMEText(email_content, 'plain', 'utf-8')
    if image_attachments or file_attachments:
        multi_email_msg = MIMEMultipart()
        multi_email_msg.attach(email_msg)
        email_msg = multi_email_msg
    # 发送者
    email_msg['From'] = formataddr(pair=(from_nickname, from_email))
    # 发送多人邮件写法
    email_msg['To'] = ','.join(receiver_list)
    if cc_list:
        # 抄送
        email_msg['Cc'] = ','.join(cc_list)
    if bcc_list:
        # 密送：建议密送地址在邮件头中隐藏
        # email_msg['Bcc'] = ','.join(bcc_list)
        pass
    email_msg['Message-id'] = email.utils.make_msgid()
    email_msg['Date'] = email.utils.formatdate()
    # 主题
    email_msg['Subject'] = subject
    if image_attachments:
        for cid, local_file in image_attachments.items():
            if os.path.exists(local_file):
                with open(local_file, 'rb') as img_f:
                    msg_img = MIMEImage(img_f.read())
                    msg_img.add_header('Content-ID', cid)
                    email_msg.attach(msg_img)
    if file_attachments:
        for file_name, local_file in file_attachments.items():
            if os.path.exists(local_file):
                with open(local_file, 'rb') as tmp_f:
                    attachment_msg = MIMEApplication(tmp_f.read())
                    attachment_msg["Content-Type"] = http_mimetype_tools.get_mimetype(local_file)[0]
                    attachment_msg.add_header('Content-Disposition', 'attachment', filename=('utf-8', '', file_name))
                    email_msg.attach(attachment_msg)
    try:
        rcpt_to = receiver_list
        if cc_list:
            rcpt_to.extend(cc_list)
        if bcc_list:
            rcpt_to.extend(bcc_list)
        send_result = smtp_client.sendmail(from_email, rcpt_to, email_msg.as_string())
        print('Email Send Success', send_result)
    except Exception as e:
        log_error(log, 'Email Send Failure', e)
    finally:
        if smtp_client:
            smtp_client.quit()


def send_html_email(smtp_client: smtplib.SMTP_SSL, from_nickname: str, from_email: str, receiver_list: list[str],
                    subject: str, email_content: str):
    try:

        # 发送纯文本格式的邮件
        email_msg = MIMEText(email_content, 'html', 'utf-8')
        # 发送者
        email_msg['From'] = formataddr(pair=(from_nickname, from_email))
        # 发送多人邮件写法
        email_msg['To'] = ','.join(receiver_list)
        # 主题
        email_msg['Subject'] = subject
        send_result = smtp_client.sendmail(from_email, receiver_list, email_msg.as_string())
        print('Email Send Success', send_result)
    except Exception as e:
        log_error(log, 'Email Send Failure', e)
    finally:
        if smtp_client:
            smtp_client.quit()


if __name__ == '__main__':
    pass
    # smtp_server = email_tools.login_smtp_server(login_email='chentiefeng521@163.com', login_password='PREYPKZJRVEJAVAH', ssl=True)
    # send_email(smtp_server, from_nickname='陈铁锋', from_email='chentiefeng521@163.com',
    #                 receiver_list=['imchentiefeng@aliyun.com', 'imchentiefeng@163.com'],
    #                 subject='hello，i just want to test',
    #                 email_content="<html><h1>人生苦短，何必执着</h1></html>", cc_list=['afenghome@aliyun.com'], bcc_list=['afengbook@aliyun.com'])
    # smtp_server = email_tools.login_smtp_server(login_email='afenghome@aliyun.com', login_password='Tie%007+521aly')
    # send_email(smtp_server, from_nickname='陈铁锋', from_email='afenghome@aliyun.com',
    #            receiver_list=['afengbook@aliyun.com'],
    #            subject='hello，i just want to test',
    #            email_content='<html><h1>人生苦短，何必执着</h1><br/><img src="cid:test" alt="logo" width="500" height="200"></html>',
    #            image_attachments={
    #                'test': r'C:\BaiduNetdiskWorkspace\教程\小程序\微信小程序\images\微信小程序开发指南\image-20231119042219753.png'},
    #            file_attachments={
    #                '小程序 - 视图与逻辑.pptx': r'C:\Users\chentiefeng\Downloads\3. 手把手教你微信小程序\微信小程序基础\微信小程序基础-资料\day03\讲义（ppt）\00_小程序 - 视图与逻辑.pptx'})



from typing import Optional, Callable, Union

from playwright.sync_api import sync_playwright, ProxySettings, Browser, BrowserContext, Page

from afeng_tools.decorator_tool.decorator_tools import run_func


def auto_input(input_type: Union[Browser, BrowserContext, Page] = Page, headless: bool = True,
               timeout: float = 1000 * 60 * 10, proxy: Optional[ProxySettings] = None, is_max: bool = False,
               storage_state: str = None):
    """注解：自动化注入 web_browser:Browser | web_context:BrowserContext | web_page:Page, 默认注入Page"""
    def auto_wrapper(func: Callable):
        def func_wrapper(*args, **kwargs):
            if input_type == Browser and 'web_browser' in kwargs:
                return run_func(func, *args, **kwargs)
            if input_type == BrowserContext and 'web_context' in kwargs:
                return run_func(func, *args, **kwargs)
            if input_type == Page and 'web_page' in kwargs:
                return run_func(func, *args, **kwargs)

            with sync_playwright() as pw:
                browser_args = []
                if is_max:
                    browser_args.append('--start-maximized')
                browser = pw.chromium.launch(headless=headless, timeout=timeout, proxy=proxy,
                                             args=browser_args)
                context = None
                page = None
                try:
                    if input_type == Browser:
                        kwargs['web_browser'] = browser
                    if input_type == BrowserContext:
                        context = browser.new_context(no_viewport=True, storage_state=storage_state)
                        kwargs['web_context'] = context
                    if input_type == Page:
                        context = browser.new_context(no_viewport=True, storage_state=storage_state)
                        page = context.new_page()
                        kwargs['web_page'] = page
                    return run_func(func, *args, **kwargs)
                finally:
                    if page:
                        page.close()
                    if context:
                        context.close()
                    browser.close()
        return func_wrapper
    return auto_wrapper



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import requests
import urlparse
import re

### M3U8 Tag
TAG_PREFIX = '#EXT'
PLAYLIST_HEADER = '#EXTM3U'
TAG_ENDLIST = '#EXT-X-ENDLIST'
TAG_KEY = '#EXT-X-KEY'
TAG_MEDIA_DURATION = "#EXTINF";
TAG_STREAM_INF = '#EXT-X-STREAM-INF'
TAG_DISCONTINUITY = '#EXT-X-DISCONTINUITY'

### M3U8 Pattern Tag
REGEX_MEDIA_DURATION = TAG_MEDIA_DURATION + ':([\\d\\.]+)\\b'

URL='http://video.yjf138.com:8091/20180812/6yl0Q2YZ/index.m3u8'

### 返回m3u8文件,不是最终的
def parse_m3u8_info(url):
    val = urlparse.urlsplit(url)
    if (val.scheme != 'http') and (val.scheme != 'https'):
        return 'Error protocol'
    result = ''
    request = requests.get(url, timeout=20)
    for line in request.iter_lines():
        result += line + '\n'
    return result

### 返回最终的m3u8文件
def parse_final_m3u8_info(url):
    val = urlparse.urlsplit(url)
    if (val.scheme != 'http') and (val.scheme != 'https'):
        return 'Error protocol'
    request = requests.get(url, timeout=20)
    result = ''
    hasStreamInf = False
    for line in request.iter_lines():
        if (line.startswith(TAG_PREFIX)):
            result += line + '\n'
            if (line.startswith(TAG_STREAM_INF)):
                hasStreamInf = True
            continue
        if (hasStreamInf):
            return parse_final_m3u8_info(get_final_url(url, line))
        hasStreamInf = False
        result += get_final_url(url, line) + '\n'
    return result

### 根据m3u8中的分片写法得到完整的url
def get_final_url(url, line):
    val = urlparse.urlsplit(url)
    hostUrl = url[0:url.index(val.netloc)+len(val.netloc)]
    baseUrl = url[0:url.rindex('/')+1]
    if (line.startswith('/')):
        tempUrl = ''
        if (line[1:].find('/') != -1):
            tempIndex = line[1:].index('/')
            tempUrl = line[0:tempIndex]
            if (url.find(tempUrl) != -1):
                tempIndex = url.index(tempUrl)
                tempUrl = url[0:tempIndex] + line
            else:
                tempUrl = hostUrl + line[1:]
            return tempUrl
        else:
            tempUrl = baseUrl + line[1:]
        return tempUrl
    if (line.startswith('http://') or line.startswith('https://')):
        return line
    return baseUrl + line

### 判断m3u8中是否存在#EXT-X-STREAM-INF
def has_ext_stream(url):
    val = urlparse.urlsplit(url)
    if (val.scheme != 'http') and (val.scheme != 'https'):
        return 'Error protocol'
    request = requests.get(url, timeout=20)
    for line in request.iter_lines():
        if (line.startswith(TAG_PREFIX)):
            if (line.startswith(TAG_STREAM_INF)):
                return True
    return False

### 判断m3u8中是否存在#EXT-X-KEY
def has_ext_key(url):
    val = urlparse.urlsplit(url)
    if (val.scheme != 'http') and (val.scheme != 'https'):
        return 'Error protocol'
    request = requests.get(url, timeout=20)
    hasStreamInf = False
    for line in request.iter_lines():
        if (line.startswith(TAG_PREFIX)):
            if (line.startswith(TAG_STREAM_INF)):
                hasStreamInf = True
            elif (line.startswith(TAG_KEY)):
                return True
            continue
        if (hasStreamInf):
            return has_ext_key(get_final_url(url, line))
        hasStreamInf = False
    return False

### 判断m3u8中是否存在#EXT-X-DISCONTINUITY
def has_ext_discontinuity(url):
    val = urlparse.urlsplit(url)
    if (val.scheme != 'http') and (val.scheme != 'https'):
        return 'Error protocol'
    request = requests.get(url, timeout=20)
    hasStreamInf = False
    for line in request.iter_lines():
        if (line.startswith(TAG_PREFIX)):
            if (line.startswith(TAG_STREAM_INF)):
                hasStreamInf = True
            elif (line.startswith(TAG_DISCONTINUITY)):
                return True
            continue
        if (hasStreamInf):
            return has_ext_key(get_final_url(url, line))
        hasStreamInf = False
    return False

### 获取M3U8文件的总时长
def get_total_duration(url):
    val = urlparse.urlsplit(url)
    if (val.scheme != 'http') and (val.scheme != 'https'):
        return 'Error protocol'
    request = requests.get(url, timeout=20)
    result = ''
    hasStreamInf = False
    totalDuration = 0
    for line in request.iter_lines():
        if (line.startswith(TAG_PREFIX)):
            if (line.startswith(TAG_STREAM_INF)):
                hasStreamInf = True
            elif (line.startswith(TAG_MEDIA_DURATION)):
                ret = parse_pattern_str(REGEX_MEDIA_DURATION, line)
                totalDuration += float(ret)
            continue
        if (hasStreamInf):
            return get_total_duration(get_final_url(url, line))
        hasStreamInf = False
    return totalDuration

def parse_pattern_str(pattern_str, str):
    matchObj = re.match(pattern_str, str)
    if (matchObj) :
        return matchObj.group(1)
    return ''

# m3u8下载的脚本
URL = 'http://video.yjf138.com:8091/20180812/6yl0Q2YZ/1500kb/hls/bm02h76314876.ts'
request = requests.get(URL, timeout=20)
with open('test.ts', 'wb') as output:
    output.write(request.content)

result = parse_m3u8_info(URL)
print('M3U8文件内容如下:\n' + result)

finalResult = parse_final_m3u8_info(URL)
print('M3U8文件最终内容如下:\n' + finalResult)

hasExtStream = has_ext_stream(URL)
print('是否存在多路流: ---> ' + str(hasExtStream))

hasExtKey = has_ext_key(URL)
print('是否存在key: ---> ' + str(hasExtKey))

hasDisContinuity = has_ext_discontinuity(URL)
print('是否存在不连续的 : ---> ' + str(hasDisContinuity))

totalDuration = get_total_duration(URL)
print('此M3U8文件的总时间: --->' + str(totalDuration))





import os.path
from typing import Any

from starlette.requests import Request
from starlette.responses import Response, RedirectResponse
from starlette.templating import Jinja2Templates, _TemplateResponse

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.fastapi_tool.fastapi_request_tools import generate_request_id
from afeng_tools.file_tool import tmp_file_tools

jinja2_templates = Jinja2Templates(directory=settings_tools.get_config(SettingsKeyEnum.server_template_path))


def create_template_response(request: Request, template_file: str, context: dict = None,
                             is_cache: bool = False, cache_path: str = None,
                             cache_file_name: str = None) -> _TemplateResponse:
    """
    创建模板响应
    :param request: Request
    :param template_file: 模板文件
    :param context: 上下文内容
    :param is_cache: 是否缓存
    :param cache_path: 缓存路径
    :param cache_file_name: 缓存文件名
    :return:
    """
    if is_cache:
        if cache_path is None:
            cache_path = os.path.join(tmp_file_tools.get_user_tmp_dir(), 'cache_html')
            os.makedirs(cache_path, exist_ok=True)
        if cache_file_name is None:
            cache_file_name = f'{generate_request_id(request)}.html'
        return create_template_response_with_cache(request=request, template_file=template_file, context=context,
                                                   cache_html_file=os.path.join(cache_path, cache_file_name))
    else:
        return create_template_response_with_cache(request=request, template_file=template_file, context=context)


def create_template_response_with_cache(request: Request, template_file: str, context: dict = None,
                                        cache_html_file: str = None,
                                        only_cache_ok: bool = True) -> _TemplateResponse:
    """
    创建模板响应
    :param request: Request
    :param template_file: 模板文件
    :param context: 上下文内容
    :param cache_html_file: 换成的html文件
    :param only_cache_ok: 只缓存200响应
    :return:
    """
    if isinstance(context, Response):
        return context
    if not context:
        context = dict()
    if 'request' not in context:
        context['request'] = request
    response = jinja2_templates.TemplateResponse(template_file, context=context)
    if cache_html_file is not None:
        if only_cache_ok:
            if response.status_code == 200:
                os.makedirs(os.path.dirname(cache_html_file), exist_ok=True)
                with open(cache_html_file, 'wb') as f:
                    f.write(response.body)
        else:
            os.makedirs(os.path.dirname(cache_html_file), exist_ok=True)
            with open(cache_html_file, 'wb') as f:
                f.write(response.body)
    return response


def create_cache_template_response(request: Request, template_file: str,
                                   context: dict[str, Any],
                                   cache_html_name: str = None) -> RedirectResponse:
    """缓存模板的html"""
    if cache_html_name is None:
        cache_html_name = hashlib_tools.calc_md5(request.url.path)
    cache_html = f'target/{cache_html_name}.html'
    app_info = request.scope.get('app_info') if request.scope else None
    app_info_code = app_info.code if app_info else 'common'
    save_template_file = os.path.join(settings_tools.get_config('server.html_save_path'),
                                      app_info_code, cache_html)
    os.makedirs(os.path.dirname(save_template_file), exist_ok=True)
    with open(save_template_file, 'wb') as f:
        if 'request' not in context:
            context['request'] = request
        response = jinja2_templates.TemplateResponse(template_file, context=context)
        f.write(response.body)
    return RedirectResponse(f'/to/{cache_html}')



from .aliyun_drive_auth_tools import *
from .aliyun_drive_download_tools import *
from .aliyun_drive_file_tools import *
from .aliyun_drive_manage_tools import *
from .aliyun_drive_share_tools import *
from .aliyun_drive_sign_tools import *
from .aliyun_drive_upload_tools import *
from .aliyun_drive_user_tools import *



"""
阿里云盘文件搜索工具
"""
from typing import List

from aligo import Aligo, BaseFile
from aligo.types.Enum import BaseFileCategory, SearchCategory


def search_aims(alipan_api: Aligo, keyword: str, category: BaseFileCategory = 'image',
                drive_id: str = None) -> List[BaseFile]:
    """
    搜索目标/标签
    :param alipan_api: [必选] Aligo
    :param keyword: [必选] 搜索的关键字
    :param category: [可选] 搜索的文件类型
    :param drive_id: [可选] 搜索的文件所在的网盘
    :return: [List[BaseFile]]
    """
    return alipan_api.search_aims(keyword=keyword, category=category, drive_id=drive_id)


def search_files(alipan_api: Aligo, name: str = None, category: SearchCategory = None,
                 drive_id: str = None) -> List[BaseFile]:
    """
    搜索文件
    :param alipan_api: [必选] Aligo
    :param name: [必选] 搜索的文件名
    :param category: [可选] 搜索的文件类型
    :param drive_id: [可选] 搜索的文件所在的网盘
    :return: [List[BaseFile]
    """
    return alipan_api.search_files(name=name, category=category, drive_id=drive_id)




from typing import Literal

import requests


class BaseWebSpider:
    @classmethod
    def handle_response(cls, response,
                        response_type: Literal['text', 'content', 'json'] = 'text',
                        response_encoding: str = 'utf-8'):
        if response.status_code == 200:
            response.encoding = response_encoding
            if response_type == 'json':
                return response.json()
            else:
                return getattr(response, response_type)

    @classmethod
    def request_get(cls, url: str, params: dict = None,
                    headers: dict = None, cookies: dict = None,
                    allow_redirects: bool = True, proxies=None,
                    response_type: Literal['text', 'content', 'json'] = 'text',
                    response_encoding: str = 'utf-8'):
        print(url)
        response = requests.get(url, params=params, headers=headers, cookies=cookies,
                                allow_redirects=allow_redirects, proxies=proxies)
        return cls.handle_response(response, response_type=response_type, response_encoding=response_encoding)


if __name__ == '__main__':
    test_spider = BaseWebSpider()
    tmp_resp = test_spider.request_get('https://quotes.toscrape.com/tag/change/page/1/')
    print(tmp_resp)


from typing import Optional, Any

from afeng_tools.pydantic_tool.model.common_models import LinkItem, EnumItem
from pydantic import BaseModel, Field

from afeng_tools.fastapi_tool.template.item import CalendarDataItem, AppinfoDataItem, Error501DataItem, \
    Error500DataItem, Error404DataItem, TemplateBaseData


class TemplateHtmlHeadData(TemplateBaseData):
    """模板head信息"""
    # 标题
    title: str
    # 描述
    description: Optional[str] = None
    # 关键字
    keyword_list: Optional[list[str]] = []
    # 作者
    author: Optional[str] = Field(default='chentiefeng')
    # favicon图标
    favicon: Optional[str] = Field(default='/favicon.ico')
    # 域信息
    origin: Optional[str] = None
    # 百度统计id
    baidu_tm_id: Optional[str] = None
    # 自定义 head部分代码列表
    custom_head_code_list: Optional[list[str]] = None


class TemplateBreadCrumbData(TemplateBaseData):
    """面包屑信息"""
    # 标题
    page_title: Optional[str] = None
    # 面包屑列表
    bread_crumb_list: Optional[list[LinkItem]] = None


class TemplateLeftNavData(TemplateBaseData):
    """左侧链接信息"""
    # 链接列表
    link_list: Optional[list[LinkItem]] = None


class TemplatePaginationAreaData(TemplateBaseData):
    """分页信息"""
    # 上一页按钮
    pre_btn: Optional[LinkItem] = None
    # 下一页按钮
    next_btn: Optional[LinkItem] = None
    # 中间数据按钮
    data_list: Optional[list[LinkItem]] = []
    # 是否显示数量区域
    is_show_count_area: Optional[bool] = True
    # 总数量
    total_count: Optional[int] = 0
    # 总页数
    total_page: Optional[int] = 0
    # 跳转到某页的地址
    jump_href: Optional[str] = None
    # 跳转页面时附加的数据数据字典
    jump_data_dict: Optional[dict[str, Any]] = None


class TemplatePageFooterData(TemplateBaseData):
    """页面底部链息"""
    # 友情链接标题
    friendly_link_title: Optional[LinkItem] = LinkItem(title='友情链接', href='/article/help/friendly_link')
    # 联系信息，如：QQ: 1640125562， 邮箱：imafengbook@aliyun.com
    contact_info: Optional[str] = None
    # 友情链接列表
    friendly_link_list: Optional[list[LinkItem]] = None
    # 底部链接列表
    footer_link_list: Optional[list[LinkItem]] = None
    # 版权链接
    copyright_link: Optional[LinkItem] = LinkItem(title='阿锋', href='/')
    # ICP备案信息，如：京ICP备2023032898号-1 京公网安备xxxx号
    icp_record_info: Optional[str] = None
    # 公安备案信息，如：京公网安备11000002000001号
    police_record_info: Optional[str] = None
    # 公安备案号：11000002000001
    police_record_code: Optional[str] = None


class TemplateTopBarData(TemplateBaseData):
    """页面顶部top bar链息"""
    have_home_btn: Optional[bool] = True
    # 应用链接列表
    app_link_list: Optional[list[LinkItem]] = None
    # 微信公众号图片
    weixin_qr_code_image: Optional[str] = '/static/image/qrcode/wx_of_qrcode.jpg'
    # 快捷链接列表
    quick_link_list: Optional[list[LinkItem]] = None


class TemplateLogoSearchData(TemplateBaseData):
    """页面顶部logo search链息"""
    # logo图片
    logo_image: Optional[str] = '/image/logo/logo.png'
    # 应用标题, 如：阿锋书屋
    app_title: Optional[str] = None
    # 查询表单提交url
    search_form_submit_url: Optional[str] = '/search'
    # 查询选项名称， 如：search_type
    search_select_type_name: Optional[str] = 'search_type'
    # 查询选项列表
    search_select_option_list: Optional[list[EnumItem]] = None


class TemplateFixNavData(TemplateBaseData):
    """页面顶部fix nav链息"""
    # 类型链接列表
    type_link_list: Optional[list[LinkItem]] = None
    # 热点链接列表
    hotspot_link_list: Optional[list[LinkItem]] = None


class TemplatePageHeaderData(TemplateBaseData):
    """页面顶部链息"""
    # topbar数据
    topbar_data: TemplateTopBarData = TemplateTopBarData()
    # logo search数据
    logo_search_data: TemplateLogoSearchData = TemplateLogoSearchData()
    # fix nav数据
    fix_nav_data: TemplateFixNavData = TemplateFixNavData()


class TemplateIndexPageHeaderData(TemplateBaseData):
    """页面顶部链息"""
    # 应用链接列表
    app_link_list: Optional[list[LinkItem]] = None
    # 微信公众号图片
    weixin_qr_code_image: Optional[str] = '/static/image/qrcode/wx_of_qrcode.jpg'
    # 快捷链接列表
    quick_link_list: Optional[list[LinkItem]] = None
    # 全部类型列表
    all_type_list: Optional[list[LinkItem]] = None


class TemplatePageSearchHeaderData(TemplateBaseData):
    """页面搜索顶部链息"""
    # 微信公众号图片
    weixin_qr_code_image: Optional[str] = '/static/image/qrcode/wx_of_qrcode.jpg'
    # 快捷链接列表
    quick_link_list: Optional[list[LinkItem]] = None
    # 查询关键字
    keyword: Optional[str] = None
    # 搜索模式
    search_model_list: Optional[list[EnumItem]] = None
    # 搜索类型
    search_type_list: Optional[list[EnumItem]] = None


class TemplateResultListData(TemplateBaseData):
    """页面结果列表链息"""
    # 结果值列表
    data_list: Optional[list[Any]] = None
    # 没有数据时的html代码
    none_html_code: Optional[str] = '暂无数据！'


class TemplateGroupListData(TemplateBaseData):
    # 查询提交url
    search_url: Optional[str] = None
    # 查询输入框placeholder
    search_placeholder: Optional[str] = None
    # 搜索值
    search_value: Optional[str] = None
    # 数据列表
    data_list: Optional[list[LinkItem]] = None


class TemplateTagListData(TemplateBaseData):
    # 查询提交url
    search_url: Optional[str] = None
    # 查询输入框placeholder
    search_placeholder: Optional[str] = None
    # 搜索值
    search_value: Optional[str] = None
    # 数据列表
    data_list: Optional[list[LinkItem]] = None


class TemplateFilterTypeAreaData(TemplateBaseData):
    # 数据列表
    data_list: Optional[list[LinkItem]] = None


class TemplateDayCalendarData(TemplateBaseData):
    # 标题
    title: Optional[str] = None
    # 初始日期
    init_date: Optional[str] = None
    # 数据列表
    data_list: Optional[list[CalendarDataItem]] = None


class TemplateTabPanelItem(BaseModel):
    # 是否激活
    is_active: Optional[bool] = False
    # 编码
    code: Optional[str] = None
    # 标题
    title: Optional[str] = None
    # html内容
    html: Optional[str] = None


class TemplateTabPanelData(TemplateBaseData):
    # 查看更多的按钮
    more_btn: Optional[LinkItem] = None
    # 子项列表
    item_list: Optional[list[TemplateTabPanelItem]] = None


class TemplateTopRankingData(TemplateBaseData):
    # 标题
    title: Optional[str] = None
    # 数据列表
    data_list: Optional[list[LinkItem]] = None


class TemplateRedirectDownloadAreaData(TemplateBaseData):
    """下载区域数据"""
    # 应用信息
    app_info: Optional[AppinfoDataItem] = None
    # 文件名
    file_name: Optional[str] = None
    # 广告列表
    ad_list: Optional[list[LinkItem]] = None
    # 下载链接
    download_url: Optional[str] = None
    # 返回链接
    back_url: Optional[str] = '/'


class TemplateRedirectAreaData(TemplateBaseData):
    """重定向区域数据"""
    # 应用信息
    app_info: Optional[AppinfoDataItem] = None
    # 广告列表
    ad_list: Optional[list[LinkItem]] = None
    # 跳转链接
    redirect_url: Optional[str] = None
    # 返回链接
    back_url: Optional[str] = '/'


class TemplateInfoAreaData(TemplateBaseData):
    """信息区域数据"""
    # 应用信息
    app_info: Optional[AppinfoDataItem] = None
    # 广告列表
    ad_list: Optional[list[LinkItem]] = None
    # 标题
    title: Optional[str] = None
    # 数据列表
    data_list: Optional[list[LinkItem]] = None


class TemplateShowQrcodeAreaData(TemplateBaseData):
    """显示二维码区域数据"""
    # 二维码图片地址
    qrcode_image_url: Optional[str] = '/static/image/qrcode/wx_of_qrcode.jpg'
    # 二维码图片标题
    qrcode_image_title: Optional[str] = None
    # 提示信息列表
    message_list: Optional[list[str]] = None


class TemplateAlertInfoAreaData(TemplateBaseData):
    """重定向区域数据"""
    # 警告图标前端代码
    alert_logo_url: Optional[
        str] = 'data:image/svg+xml;base64,PHN2ZyB0PSIxNzEzODQ4NjkwODEzIiBjbGFzcz0iaWNvbiIgdmlld0JveD0iMCAwIDEwMjQgMTAyNCIgdmVyc2lvbj0iMS4xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHAtaWQ9IjE0NTYiIHdpZHRoPSIyMDAiIGhlaWdodD0iMjAwIj48cGF0aCBkPSJNNTEyLjIxNjgzIDAuMDM4MTU0QzIyOS45NDM4MTMgMC4wMzgxNTQgMS4xMTg5NDkgMjI5LjExODQwNCAxLjExODk0OSA1MTEuNzA2NDU3IDEuMTE4OTQ5IDc5NC4yOTQ1MTEgMjI5Ljk0MzgxMyAxMDIzLjM3NDc2MyA1MTIuMjE2ODMgMTAyMy4zNzQ3NjMgNzk0LjQ4OTg0NSAxMDIzLjM3NDc2MyAxMDIzLjMxNDcxMSA3OTQuMjk0NTExIDEwMjMuMzE0NzExIDUxMS43MDY0NTcgMTAyMy4zMTQ3MTEgMjI5LjExODQwNCA3OTQuNDg5ODQ1IDAuMDM4MTU0IDUxMi4yMTY4MyAwLjAzODE1NEw1MTIuMjE2ODMgMC4wMzgxNTRaTTUxMi4yMTY4MyA4MzEuNTAwNzU0QzQ3Ni45MzE2ODEgODMxLjUwMDc1NCA0NDguMzI5NzQxIDgwMi44NjY4OTMgNDQ4LjMyOTc0MSA3NjcuNTQyMzYzIDQ0OC4zMjk3NDEgNzMyLjIxNzgzMyA0NzYuOTMxNjgxIDcwMy41ODM5NzIgNTEyLjIxNjgzIDcwMy41ODM5NzIgNTQ3LjUwMTk3OSA3MDMuNTgzOTcyIDU3Ni4xMDM5MTkgNzMyLjIxNzgzMyA1NzYuMTAzOTE5IDc2Ny41NDIzNjMgNTc2LjEwMzkxOSA4MDIuODY2ODkzIDU0Ny41MDE5NzkgODMxLjUwMDc1NCA1MTIuMjE2ODMgODMxLjUwMDc1NEw1MTIuMjE2ODMgODMxLjUwMDc1NFpNNTc2LjEwNTA4NSA1NzUuNjY2MDE3QzU3Ni4xMDUwODUgNjEwLjk5MDU0NiA1NDcuNTAzMTQ3IDYzOS42MjQ0MDggNTEyLjIxNzk5OCA2MzkuNjI0NDA4IDQ3Ni45MzI4NDggNjM5LjYyNDQwOCA0NDguMzMwOTA4IDYxMC45OTA1NDYgNDQ4LjMzMDkwOCA1NzUuNjY2MDE3TDQ0OC4zMzA5MSAyNTUuODcyODlDNDQ4LjMzMDkxIDIyMC41NDgzNiA0NzYuOTMyODQ5IDE5MS45MTQ0OTggNTEyLjIxNzk5OSAxOTEuOTE0NDk4IDU0Ny41MDMxNDggMTkxLjkxNDQ5OCA1NzYuMTA1MDkxIDIyMC41NDgzNiA1NzYuMTA1MDkxIDI1NS44NzI4OUw1NzYuMTA1MDg1IDU3NS42NjYwMTcgNTc2LjEwNTA4NSA1NzUuNjY2MDE3WiIgZmlsbD0iI0RDNEYyQyIgcC1pZD0iMTQ1NyI+PC9wYXRoPjwvc3ZnPg=='
    # 警告标题
    alert_title: Optional[str] = '警告'
    # 提示信息列表
    alert_message_list: list[str]


class TemplateError404AreaData(TemplateBaseData, Error404DataItem):
    """错误404数据"""
    pass


class TemplateError500AreaData(TemplateBaseData, Error500DataItem):
    """错误500数据"""
    pass


class TemplateError501AreaData(TemplateBaseData, Error501DataItem):
    """错误501数据"""
    pass



from typing import Any
from sqlalchemy.orm import Session

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService
from afeng_tools.pydantic_tool.model.common_models import LinkItem
from afeng_tools.sqlalchemy_tools.decorator.sqlalchemy_session_decorator import auto_db


class TagPoService(PoService):
    """
    使用示例：tag_po_service = TagPoService(app_info.db_code, TagInfoPo)
    """
    _table_name_ = "tb_tag_info"

    def get_by_code(self, type_code: str, code: str) -> Any:
        return self.get(self.model_type.type_code == type_code, self.model_type.code == code)

    @classmethod
    def convert_to_link_item(cls, tag_po, is_active: bool = False, tag_href_prefix: str = '/tag') -> LinkItem:
        return LinkItem(
            target='_blank',
            title=tag_po.title,
            href=f'{tag_href_prefix}/{tag_po.code}',
            code=tag_po.code,
            description=tag_po.description,
            is_active=is_active)

    def query_tag_by_relation_type_code(self, type_code: str, type_value: str, tag_relation_info_po_class) -> list:
        """
        通过 type_code和type_value查询关联的标签
        :param type_code:
        :param type_value:
        :param tag_relation_info_po_class:TagRelationInfoPo类
        :return:
        """
        @auto_db(db_code=self.db_code)
        def _query_by_type_code(_type_code: str, _type_value: str, _tag_relation_info_po_class, db: Session = None) -> list:
            db_query = (db.query(self.model_type, _tag_relation_info_po_class)
                        .select_from(self.model_type).join(_tag_relation_info_po_class,
                                                           _tag_relation_info_po_class.tag_code == self.model_type.code,
                                                           isouter=True))
            db_query = (
                db_query.filter(_tag_relation_info_po_class.type_code == _type_code,
                                _tag_relation_info_po_class.type_value == _type_value)
                .order_by(_tag_relation_info_po_class.order_num.desc()))
            query_result = db_query.all()
            if query_result:
                return [type_info_po for type_info_po, tag_relation_po in query_result if type_info_po]
            return []

        return _query_by_type_code(_type_code=type_code, _type_value=type_value,
                                   _tag_relation_info_po_class=tag_relation_info_po_class)


class TagRelationPoService(PoService):
    """
    使用示例：tag_relation_po_service = TagRelationPoService(app_info.db_code, TagRelationInfoPo)
    """
    _table_name_ = "tb_tag_relation_info"

    def query_by_type_code(self, type_code: str) -> list[Any]:
        return self.get(self.model_type.type_code == type_code)

    def get_by_tag_code(self, type_code: str, tag_code: str) -> Any:
        return self.get(self.model_type.type_code == type_code, self.model_type.tag_code == tag_code)



from typing import Type

from afeng_tools.encryption_tool import hashlib_tools
from pydantic import BaseModel
from starlette.datastructures import FormData
from starlette.requests import Request

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.convert_tool.type_convert_tools import convert_value_to_type
from afeng_tools.http_tool import http_request_tools


def generate_request_id(request: Request) -> str:
    """生成request的唯一id"""
    return hashlib_tools.calc_unique_id(data_list=[
        http_request_tools.is_mobile(request.headers.get('user-agent')),
        request.method,
        request.url.path,
        request.query_params,
    ], split_chat='||')


def form_params_convert_to_model(form_model_class: Type[BaseModel]):
    """将表单参数转换为pydantic的Model对象"""

    async def func_wrap(request: Request):
        body = await request.form()
        if body:
            model_value_dict = dict()
            field_dict = form_model_class.model_fields
            for tmp_key, tmp_value in body.items():
                if tmp_key in field_dict:
                    model_value_dict[tmp_key] = convert_value_to_type(tmp_value, field_dict.get(tmp_key).annotation)
            return form_model_class(**model_value_dict)

    return func_wrap


async def do_handle_form_array_item(form_data: FormData) -> FormData:
    """处理form表单提交的数组"""
    new_form_data_list = []
    for tmp_key in form_data.keys():
        if tmp_key.endswith('[]'):
            new_form_data_list.append((tmp_key.removesuffix('[]'), form_data.getlist(tmp_key)))
        else:
            new_form_data_list.append((tmp_key, form_data.get(tmp_key)))
    return FormData(new_form_data_list)


def get_request_app_info(request: Request, app_dict: dict[str, AppInfo]) -> AppInfo:
    """获取请求的AppInfo"""
    if hasattr(request, 'scope') and request.scope:
        app_info = request.scope.get('app_info')
        if app_info:
            return app_info
    request_path = request.url.path
    if app_dict:
        for app_info in app_dict.values():
            if request_path.startswith(app_info.prefix + '/'):
                request.scope['app_info'] = app_info
                return app_info



from enum import Enum

from afeng_tools.pydantic_tool.model.common_models import EnumItem


class ApiErrorEnum(Enum):
    hit_frequency_limit = EnumItem(value='hit_frequency_limit', title='接口请求过于频繁，注意控制',
                                   description='命中接口频控')
    no_permission_to_access = EnumItem(value='no_permission_to_access', title='没有访问的权限')
    file_does_not_exist = EnumItem(value='file_does_not_exist', title='文件不存在')
    invalid_request = EnumItem(value='invalid_request',
                               title='请求缺少某个必需参数，包含一个不支持的参数或参数值，或者格式不正确')
    invalid_client = EnumItem(value='unknown_client',
                              title='“client_id”、“client_secret”参数无效')
    invalid_grant = EnumItem(value='grant_is_revoked',
                             title='授权是是无效的',
                             description='提供的Access Grant是无效的、过期的或已撤销的，例如，Authorization Code无效(一个授权码只能使用一次)、Refresh Token无效、redirect_uri与获取Authorization Code时提供的不一致、Devie Code无效(一个设备授权码只能使用一次)等。')
    unauthorized_client = EnumItem(value='client_is_not_authorized',
                                   title='应用没有被授权')
    unsupported_grant_type = EnumItem(value='grant_type_is_not_supported',
                                      title='不支持该授权类型')
    invalid_scope = EnumItem(value='the_requested_scope_is_exceeds_the_scope_granted',
                             title='请求的“scope”参数是无效的、未知的、格式不正确的、或所请求的权限范围超过了数据拥有者所授予的权限范围')

    expired_token = EnumItem(value='expired_token',
                             title='提供的Token已过期')
    redirect_uri_mismatch = EnumItem(value='invalid_redirect_uri',
                                     title='“redirect_uri”所在的根域与开发者注册应用时所填写的根域名不匹配')
    unsupported_response_type = EnumItem(value='the_response_type_is_not_supported',
                                         title='“response_type”参数值不支持，或者应用已经主动禁用了对应的授权模式')
    slow_down = EnumItem(value='the_request_is_polling_too_frequently',
                         title='Device Flow中，换取Access Token的接口过于频繁，两次尝试的间隔应大于5秒。')
    authorization_pending = EnumItem(value='user_has_not_yet_completed_the_authorization',
                                     title='用户还没有对Device Code完成授权操作。')
    authorization_declined = EnumItem(value='user_has_declined_the_authorization',
                                      title='用户拒绝了对Device Code的授权操作。')
    unknown_error = EnumItem(value='unknown_error',
                             title='未知错误，如果频繁发生此错误，请联系管理员')
    service_temporarily_unavailable = EnumItem(value='service_temporarily_unavailable',
                                               title='服务暂时不可用')
    unsupported_openapi_method = EnumItem(value='unsupported_openapi_method',
                                          title='访问URL错误，该接口不能访问')
    request_limit_reached = EnumItem(value='open_api_request_limit_reached',
                                     title='访问该接口的QPS达到上限')
    unauthorized_client_ip_address = EnumItem(value='unauthorized_client_ip_address',
                                              title='访问的客户端IP不在白名单内')
    daily_request_limit_reached = EnumItem(value='open_api_daily_request_limit_reached',
                                           title='访问该接口超过每天的访问限额')
    qps_request_limit_reached = EnumItem(value='open_api_qps_request_limit_reached',
                                         title='访问该接口超过QPS限额')
    total_request_limit_reached = EnumItem(value='open_api_total_request_limit_reached',
                                           title='访问该接口超过总量限额')
    invalid_parameter = EnumItem(value='invalid_parameter',
                                 title='没有获取到合法参数')
    access_token_invalid = EnumItem(value='access_token_invalid_or_no_longer_valid',
                                    title='token不合法')
    access_token_expired = EnumItem(value='access_token_expired',
                                    title='token已过期')
    no_permission_to_access_user_mobile = EnumItem(value='no_permission_to_access_user_mobile',
                                                   title='没有权限获取用户手机号')



"""
用户工具
pip install aligo -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os.path
import time
import traceback
from typing import List, Callable

from afeng_tools.cache_tool import cache_memory_tools
from aligo import Aligo, ShareLinkExtractCodeResponse, GetShareTokenResponse, BatchShareFileSaveToDriveResponse, \
    CreateShareLinkResponse, CancelShareLinkResponse, BatchSubResponse, ShareLinkSchema, GetShareInfoResponse, \
    BaseShareFile, GetShareLinkVideoPreviewPlayInfoResponse, BaseFile, PrivateShareResponse
from aligo.types.Enum import GetShareLinkListOrderBy, OrderDirection, SearchFileOrderBy, CheckNameMode
from loguru import logger

from .core import alipan_file_tools
from .core.custom_aligo import CustomAligo


def create_share(alipan_api: Aligo, file_id: str | list[str],
                 share_pwd: str = None,
                 expiration: str = None,
                 drive_id: str = None,
                 description: str = None) -> CreateShareLinkResponse:
    """
    创建分享
    :param alipan_api:
    :param file_id: [必选] 文件id
    :param share_pwd: [可选] 分享密码，默认：None，表示无密码
    :param expiration: [可选] 有效期，utc时间字符串：YYYY-MM-DDTHH:mm:ss.SSSZ ，如：'2021-12-01T00:00:00.000Z'
    :param drive_id: [可选] 所属网盘id
    :param description: [可选] 描述
    :return:
    """
    if isinstance(file_id, list):
        return alipan_api.share_files(file_id, share_pwd=share_pwd,
                                      expiration=expiration,
                                      drive_id=drive_id,
                                      description=description)
    return alipan_api.share_file(file_id, share_pwd=share_pwd,
                                 expiration=expiration,
                                 drive_id=drive_id,
                                 description=description)


def get_share_list(alipan_api: Aligo, order_by: GetShareLinkListOrderBy = 'created_at',
                   order_direction: OrderDirection = 'DESC',
                   include_canceled: bool = False) -> List[ShareLinkSchema]:
    """
    获取分享列表
    :param alipan_api:
    :param order_by: [可选] 排序字段，默认：created_at
        :param order_direction: [可选] 排序方向，默认：DESC
        :param include_canceled: [可选] 是否包含已取消的分享，默认：False
        :return: [List[ShareLinkSchema]]
    :return:
    """
    return alipan_api.get_share_list(order_by=order_by, order_direction=order_direction,
                                     include_canceled=include_canceled)


def cancel_share(alipan_api: Aligo, share_id: str) -> CancelShareLinkResponse:
    """取消分享"""
    return alipan_api.cancel_share(share_id)


def batch_cancel_share(alipan_api: Aligo, share_id_list: List[str]) -> List[BatchSubResponse]:
    """批量取消分享"""
    return alipan_api.batch_cancel_share(share_id_list)


def get_share_list(alipan_api: Aligo,
                   order_by: GetShareLinkListOrderBy = 'created_at',
                   order_direction: OrderDirection = 'DESC',
                   include_canceled: bool = False) -> List[ShareLinkSchema]:
    """
    获取分享列表
    :param alipan_api:
    :param order_by: [可选] 排序字段，默认：created_at
    :param order_direction: [可选] 排序方向，默认：DESC
    :param include_canceled: [可选] 是否包含已取消的分享，默认：False
    :return: [List[ShareLinkSchema]]
    :return:
    """
    return alipan_api.get_share_list(order_by=order_by,
                                     order_direction=order_direction,
                                     include_canceled=include_canceled)


def get_share_info(alipan_api: Aligo, share_id: str) -> GetShareInfoResponse:
    """
    获取分享信息
    :param alipan_api:
    :param share_id: 分享id
    :return: GetShareInfoResponse
    """
    return alipan_api.get_share_info(share_id)


def get_share_info_by_link(alipan_api: Aligo, share_msg: str) -> ShareLinkExtractCodeResponse:
    """
    通过链接获取分享信息
    :param alipan_api: Aligo
    :param share_msg: 分享信息
                如：「心灵奇旅 4K 原盘 REMUX」https://www.aliyundrive.com/s/FYoddEbsSwV
                如： https://www.aliyundrive.com/s/LkUS6BrconT/folder/62074d95b39e519f60be4f73b68ac493e9977a90
    :return: 分享信息，如：result.share_id, result.share_pwd
    """
    return alipan_api.share_link_extract_code(share_msg)


def get_share_token(alipan_api: Aligo, share_id: str, share_pwd: str = '') -> GetShareTokenResponse:
    """获取share_token"""
    return alipan_api.get_share_token(share_id, share_pwd=share_pwd)


def get_share_token_with_cache(alipan_api: Aligo, share_id: str, share_pwd: str = '',
                               cache_group_code: str = 'share_token',
                               space_value: int = 100) -> GetShareTokenResponse:
    """获取缓存的share_token"""
    token_info = cache_memory_tools.get_time_cache(cache_group_code, share_id)
    if token_info:
        add_time, share_token = token_info
        if time.time() - add_time < space_value:
            return share_token
    share_token = get_share_token(alipan_api, share_id, share_pwd=share_pwd)
    cache_memory_tools.add_time_cache(cache_group_code, share_id, share_token)
    return share_token


def get_share_file_list(alipan_api: Aligo, share_token: GetShareTokenResponse,
                        parent_file_id: str = 'root', parent_path: str = '/',
                        is_recursion: bool = False, recursion_deep: int = 999999999) -> List[BaseShareFile]:
    """
    获取分享文件列表
    :param alipan_api:
    :param share_token:
    :param parent_file_id:
     :param parent_path: 上级文件路径（这个不是阿里云盘必须的，只是效仿百度网盘的路径表示方式）
    :param is_recursion: 是否递归深入获取
    :param recursion_deep: 递归的层级，默认999999999，几乎代表无限递归了
    :return:
    """
    result_file_list = alipan_api.get_share_file_list(share_token, parent_file_id=parent_file_id)
    if is_recursion and recursion_deep > 0:
        for tmp_file in result_file_list:
            tmp_file.path = f'/{parent_path.removeprefix("/").removesuffix("/")}/{tmp_file.name}'
            if alipan_file_tools.is_folder(tmp_file):
                child_file_list = get_share_file_list(alipan_api, share_token,
                                                      parent_file_id=tmp_file.file_id,
                                                      parent_path=tmp_file.path,
                                                      is_recursion=is_recursion,
                                                      recursion_deep=recursion_deep - 1)
                if child_file_list:
                    result_file_list.extend(child_file_list)
    else:
        for tmp_file in result_file_list:
            tmp_file.path = f'/{parent_path.removeprefix("/").removesuffix("/")}/{tmp_file.name}'
    return result_file_list


def iterate_handle_share_file_list(alipan_api: Aligo, share_token: GetShareTokenResponse,
                                   iterate_handle_func: Callable[[Aligo, GetShareTokenResponse, BaseShareFile], bool],
                                   parent_file_id: str = 'root', parent_path: str = '/',
                                   is_recursion: bool = False, recursion_deep: int = 999999999):
    """
    迭代处理分享文件列表
    :param alipan_api:
    :param share_token:
    :param iterate_handle_func: 迭代处理函数， 参数（alipan_api: Aligo, share_token: GetShareTokenResponse,ali_file:BaseShareFile）
            - 如果是文件夹， 当返回值为True时，则继续向下递归，如果为false，则不再递归
    :param parent_file_id:
     :param parent_path: 上级文件路径（这个不是阿里云盘必须的，只是效仿百度网盘的路径表示方式）
    :param is_recursion: 是否递归深入获取
    :param recursion_deep: 递归的层级，默认999999999，几乎代表无限递归了
    :return:
    """
    file_list = alipan_api.get_share_file_list(share_token, parent_file_id=parent_file_id)
    if is_recursion and recursion_deep > 0:
        for tmp_file in file_list:
            tmp_file.path = f'/{parent_path.removeprefix("/").removesuffix("/")}/{tmp_file.name}'
            if iterate_handle_func(alipan_api, share_token, tmp_file):
                if alipan_file_tools.is_folder(tmp_file):
                    iterate_handle_share_file_list(alipan_api, share_token,
                                                   iterate_handle_func=iterate_handle_func,
                                                   parent_file_id=tmp_file.file_id,
                                                   parent_path=tmp_file.path,
                                                   is_recursion=is_recursion,
                                                   recursion_deep=recursion_deep - 1)
    else:
        for tmp_file in file_list:
            tmp_file.path = f'/{parent_path.removeprefix("/").removesuffix("/")}/{tmp_file.name}'
            iterate_handle_func(alipan_api, share_token, tmp_file)


def get_share_file(alipan_api: Aligo, share_token: GetShareTokenResponse, file_id: str) -> BaseShareFile:
    """获取分享文件"""
    return alipan_api.get_share_file(file_id=file_id, share_token=share_token)


def save_share_file_to_pan(alipan_api: Aligo, share_token: GetShareTokenResponse,
                           file_id: str,
                           to_parent_file_id: str = 'root',
                           new_name: str = None,
                           to_drive_id: str = None) -> List[BatchShareFileSaveToDriveResponse]:
    """
    保存分享文件到云盘
    :param alipan_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param file_id: 文件id
    :param to_parent_file_id: 目标父文件夹id，默认为根目录
    :param new_name: 新文件名
    :param to_drive_id: 目标网盘id，默认为当前网盘
    :return:
    """
    return alipan_api.share_file_saveto_drive(file_id=file_id, share_token=share_token,
                                              to_parent_file_id=to_parent_file_id, to_drive_id=to_drive_id,
                                              new_name=new_name)


def save_share_files_to_pan(alipan_api: Aligo, share_token: GetShareTokenResponse,
                            file_id_list: List[str],
                            to_parent_file_id: str = 'root',
                            auto_rename: bool = True,
                            to_drive_id: str = None) -> List[BatchShareFileSaveToDriveResponse]:
    """
    批量保存分享文件到云盘
    :param alipan_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param file_id_list: 文件id
    :param to_parent_file_id: 目标父文件夹id，默认为根目录
    :param auto_rename:
    :param to_drive_id: 目标网盘id，默认为当前网盘
    :return:
    """
    return alipan_api.batch_share_file_saveto_drive(file_id_list=file_id_list, share_token=share_token,
                                                    to_parent_file_id=to_parent_file_id,
                                                    auto_rename=auto_rename,
                                                    to_drive_id=to_drive_id)


def batch_save_share_file_to_pan(alipan_api: Aligo, share_token: GetShareTokenResponse,
                                 share_parent_file_id: str = 'root',
                                 to_parent_file_id: str = 'root',
                                 auto_rename: bool = True,
                                 to_drive_id: str = None) -> List[BatchShareFileSaveToDriveResponse]:
    """
    批量保存分享文件到云盘
     :param alipan_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param share_parent_file_id:
    :param to_parent_file_id:
    :param auto_rename:
    :param to_drive_id:

    :return: List[BatchShareFileSaveToDriveResponse]
    """
    # 如果遇到分享文件非常多，此段代码运行完成后，不会立马看到所有文件，可能需要几个小时才能陆续保存完成
    # 在网页保存需要几个小时，用这个一下就可以了，阿里云服务器会处理，不用等待
    return alipan_api.share_file_save_all_to_drive(share_token,
                                                   to_parent_file_id=to_parent_file_id,
                                                   parent_file_id=share_parent_file_id,
                                                   auto_rename=auto_rename,
                                                   to_drive_id=to_drive_id)


def search_share_files(alipan_api: Aligo, share_token: GetShareTokenResponse, keyword: str,
                       order_by: SearchFileOrderBy = 'name',
                       order_direction: OrderDirection = 'DESC') -> List[BaseShareFile]:
    """
    在分享中搜索文件
    :param alipan_api:
    :param share_token:
    :param keyword:
    :param order_by:
    :param order_direction:
    :return:
    """
    return alipan_api.search_share_files(keyword=keyword, share_token=share_token,
                                         order_by=order_by, order_direction=order_direction)


def download_share_file(alipan_api: CustomAligo, share_token: GetShareTokenResponse,
                        share_file: BaseShareFile,
                        local_folder: str = '.') -> str:
    """
    下载分享文件
    :param alipan_api:
    :param share_token:
    :param share_file:
    :param local_folder:
    :return:
    """
    if alipan_file_tools.is_folder(share_file):
        local_folder = os.path.join(local_folder, share_file.name)

        tmp_file_list = get_share_file_list(alipan_api, share_token, parent_file_id=share_file.file_id,
                                            is_recursion=False)
        if tmp_file_list:
            for tmp_file in tmp_file_list:
                download_share_file(alipan_api, share_token, share_file=tmp_file,
                                    local_folder=local_folder)
        return local_folder
    else:
        if share_token is None:
            share_token = get_share_token_with_cache(alipan_api, share_file.share_id)
        cache_folder_file = alipan_api.get_folder_by_path(path='/cache', create_folder=True)
        save_result = alipan_api.share_file_saveto_drive(file_id=share_file.file_id,
                                                         share_token=share_token,
                                                         to_parent_file_id=cache_folder_file.file_id)
        if hasattr(save_result, 'message'):
            raise Exception(save_result.message)
        pan_file_id = save_result.file_id
        share_file.pan_file_id = pan_file_id
        download_resp = alipan_api.get_download_url(file_id=save_result.file_id)
        share_file.md5 = download_resp.content_hash
        share_file.url = download_resp.url
        share_file.download_url = download_resp.url
        download_url = download_resp.url
        try:
            os.makedirs(local_folder, exist_ok=True)
            local_file = os.path.join(local_folder, share_file.name)
            local_file = alipan_api.download_file(file_path=local_file, url=download_url)
            if os.path.exists(local_file):
                if os.stat(local_file).st_size == 350:
                    os.remove(local_file)
                    download_url = alipan_api.get_download_url(file_id=pan_file_id)
                    local_file = alipan_api.download_file(file_path=local_file, url=download_url)
                return local_file
        except Exception as ex:
            logger.error(f'{ex}\n {traceback.format_exc()}')
        finally:
            alipan_api.delete_file(file_id=pan_file_id)


def create_private_share_file(alipan_api: Aligo, file_id: str, drive_id: str = None) -> PrivateShareResponse:
    """APP 中的快传，有效期 24 小时，只能一个人保存"""
    return alipan_api.private_share_file(file_id, drive_id=drive_id)


def create_private_share_files(alipan_api: Aligo, file_id_list: List[str],
                               drive_id: str = None) -> PrivateShareResponse:
    return alipan_api.private_share_files(file_id_list, drive_id=drive_id)


def get_share_link_video_preview_play_info(alipan_api: Aligo, share_token: GetShareTokenResponse,
                                           video_file_id: str,
                                           drive_id: str) -> GetShareLinkVideoPreviewPlayInfoResponse:
    return alipan_api.get_share_link_video_preview_play_info(file_id=video_file_id,
                                                             drive_id=drive_id,
                                                             x_share_token=share_token)


def create_aligo_file_share(alipan_api: Aligo, file: BaseFile) -> str:
    """创建Aligo文件分享"""
    return alipan_api.share_file_by_aligo(file)


def create_aligo_files_share(alipan_api: Aligo, files: List[BaseFile]) -> str:
    """创建Aligo文件分享"""
    return alipan_api.share_files_by_aligo(files)


def create_aligo_folder_share(alipan_api: Aligo, file_id: str) -> str:
    """创建Aligo文件夹分享"""
    return alipan_api.share_folder_by_aligo(file_id)


def save_aligo_share(alipan_api: Aligo, aligo_share_data: str) -> list:
    """保存Aligo分享"""
    return alipan_api.save_files_by_aligo(aligo_share_data)



import os
from typing import List, Optional, Callable, Union

from aligo import Aligo, BaseFile, BatchSubResponse, FolderSizeInfo, CreateFileResponse, GetFilePathResponse
from aligo.types.Enum import CheckNameMode

from .core import alipan_file_tools


def get_folder_size_info(alipan_api: Aligo, file_id: str, drive_id: str = None) -> FolderSizeInfo:
    """获取文件夹大小信息"""
    return alipan_api.get_folder_size_info(file_id=file_id, drive_id=drive_id)


def get_file(alipan_api: Aligo, file_id: str, drive_id: str = None) -> BaseFile:
    """获取文件信息"""
    return alipan_api.get_file(file_id=file_id, drive_id=drive_id)


def get_file_path(alipan_api: Aligo, file_id: str, drive_id: str = None) -> List[BaseFile]:
    """
    获取文件路径
    :param alipan_api:
    :param file_id:
    :param drive_id:
    :return: 路径从底层到高层的列表，如：/cache/afeng/test/tmp.png 返回路径的顺序是：[test, afeng, cache]
    """
    resp = alipan_api.get_path(file_id=file_id, drive_id=drive_id)
    if isinstance(resp, GetFilePathResponse):
        return resp.items


def get_file_list(alipan_api: Aligo, parent_file_id: str = 'root',
                  drive_id: str = None, parent_path: str = '/',
                  is_recursion: bool = False, recursion_deep: int = 999999999) -> List[BaseFile]:
    """列出路径下的文件"""
    result_file_list = alipan_api.get_file_list(parent_file_id=parent_file_id, drive_id=drive_id)
    if is_recursion and recursion_deep > 0:
        for tmp_file in result_file_list:
            tmp_file.path = f'/{parent_path.removeprefix("/").removesuffix("/")}/{tmp_file.name}'
            if alipan_file_tools.is_folder(tmp_file):
                child_file_list = get_file_list(alipan_api,
                                                parent_file_id=tmp_file.file_id,
                                                drive_id=drive_id,
                                                parent_path=tmp_file.path,
                                                is_recursion=is_recursion, recursion_deep=recursion_deep - 1)
                if child_file_list:
                    result_file_list.extend(child_file_list)
    else:
        for tmp_file in result_file_list:
            tmp_file.path = f'/{parent_path.removeprefix("/").removesuffix("/")}/{tmp_file.name}'
    return result_file_list


def batch_get_files(alipan_api: Aligo, file_id_list: List[str],
                    drive_id: str = None) -> List[BatchSubResponse]:
    """批量获取文件信息"""
    return alipan_api.batch_get_files(file_id_list=file_id_list, drive_id=drive_id)


def iterate_handle_all_file(alipan_api: Aligo,
                            iterate_handle_func: Callable[[Aligo, BaseFile], None],
                            parent_file_id: str = 'root',
                            parent_path: str = '/',
                            drive_id: str = None, ):
    """
    迭代处理所有文件（会递归处理所有文件）
    :param alipan_api:
    :param iterate_handle_func: 迭代处理函数， 参数（alipan_api: Aligo, share_token: GetShareTokenResponse,ali_file:BaseShareFile）
            - 如果是文件夹， 当返回值为True时，则继续向下递归，如果为false，则不再递归
    :param parent_file_id:
     :param parent_path: 上级文件路径（这个不是阿里云盘必须的，只是效仿百度网盘的路径表示方式）
     :param drive_id:
    :return:
    """

    def _file_handle(pan_path: str, pan_file: BaseFile):
        pan_file.path = f'/{pan_path.replace(os.sep, "/").removeprefix("/").removesuffix("/")}/{pan_file.name}'
        iterate_handle_func(alipan_api, pan_file)

    alipan_api.walk_files(callback=_file_handle,
                          parent_file_id=parent_file_id,
                          drive_id=drive_id,
                          _path=parent_path)


def get_file_by_path(alipan_api: Aligo, file_path: str = '/',
                     parent_file_id: str = 'root',
                     check_name_mode: CheckNameMode = 'refuse',
                     drive_id: str = None) -> Optional[BaseFile]:
    """通过路径获取文件信息"""
    return alipan_api.get_file_by_path(path=file_path,
                                       parent_file_id=parent_file_id,
                                       check_name_mode=check_name_mode, drive_id=drive_id)


def get_folder_by_path(alipan_api: Aligo, file_path: str = '/',
                       parent_file_id: str = 'root',
                       create_folder: bool = False,
                       check_name_mode: CheckNameMode = 'refuse',
                       drive_id: str = None) -> Union[BaseFile, CreateFileResponse, None]:
    """通过路径获取文件信息"""
    return alipan_api.get_folder_by_path(path=file_path,
                                         parent_file_id=parent_file_id,
                                         create_folder=create_folder,
                                         check_name_mode=check_name_mode, drive_id=drive_id)



from typing import Optional
from sqlalchemy.pool import QueuePool
from pydantic import BaseModel


class DatabaseInfoItem(BaseModel):
    """数据库信息项"""
    # 数据库连接地址, 格式：dialect+driver://username:password@host:port/database， 示例：postgresql://user:pass@localhost/mydatabase
    database_uri: str
    # 是否打印sql， 如果设置为 True，SQLAlchemy 将在控制台输出所有执行的 SQL 语句。
    echo_sql: Optional[bool] = False
    pool_class: Optional[type] = QueuePool
    # 当设置为 True 时，SQLAlchemy 将输出有关连接池操作的日志信息。
    echo_pool: Optional[bool | str] = False
    # 指定连接池的大小。这是连接池中可以保持的活动连接数，连接池中的最小和最大连接数
    pool_size: Optional[int] = 5
    # 指定在连接池耗尽时，可以额外创建的连接数。（在达到pool_size后可以额外创建的连接数）
    pool_max_overflow: Optional[int] = 10
    # 回收时间(秒), 指定连接在被返回到连接池之前应该保持多长时间的打开状态。这有助于确保连接不会因为长时间未使用而被数据库服务器关闭。
    pool_recycle: Optional[int] = 3600
    # 指定当从连接池中获取连接时应该等待多长时间(秒)。如果在这个时间内没有可用的连接，将会抛出一个异常。
    pool_timeout: Optional[int] = 30
    # 预ping检查
    pool_pre_ping: Optional[bool] = True
    # 是否（后进先出）
    pool_use_lifo: Optional[bool] = False
    # 是否自动提交
    auto_commit: Optional[bool] = False
    # 是否自动flush
    auto_flush: Optional[bool] = False
    # 超时是否自动提交
    expire_on_commit: Optional[bool] = False



from typing import Any, Optional

from pydantic import BaseModel


class PageTemplateItem(BaseModel):
    """页面模板项"""
    is_mobile: Optional[bool] = False
    # 页面html中head标签元素
    html_head_template: Optional[str] = None
    # 页面头部模板
    page_header_template: Optional[str] = None
    # 页面底部模板
    page_footer_template: Optional[str] = None
    data_dict: Optional[dict[str, Any]] = None


class ArticleTemplateItem(PageTemplateItem):
    # 面包屑模板
    bread_crumb_template: Optional[str] = None
    # 左侧导航模板
    left_nav_template: Optional[str] = None
    # 文章标题
    title: Optional[str] = None
    # 文章副标题
    sub_title: Optional[str] = None
    # 文章内容
    content: Optional[str] = None
    # 元数据
    meta_data: Optional[dict[str, Any]] = None



from typing import Callable, Optional, Union, List

import scrapy


def create_request(url: str, callback: Optional[Callable] = None,
                   method: str = "GET",
                   headers: Optional[dict] = None,
                   cookies: Optional[Union[dict, List[dict]]] = None,
                   body: Optional[Union[bytes, str]] = None,
                   encoding: str = "utf-8",
                   cb_kwargs: Optional[dict] = None) -> scrapy.Request:
    """
    创建request请求
    :param url: 请求地址
    :param callback: 回调
    :param method: 请求方法
    :param headers: 请求头
    :param cookies: Cookie
    :param body: 提交内容
    :param encoding: 编码
    :param cb_kwargs: 参数
    :return: scrapy.Request
    """
    return scrapy.Request(url=url,
                          method=method,
                          headers=headers,
                          cookies=cookies,
                          body=body,
                          encoding=encoding,
                          callback=callback,
                          cb_kwargs=cb_kwargs,
                          dont_filter=True)



from typing import Literal
import requests

from config import settings

from afeng_tools.application_tool import settings_tools
from afeng_tools.weixin_tool.token import wx_token_tools


# 永久二维码
def create_qrcode(access_token: str, scene_value: str | int,
                  action_name: Literal['QR_SCENE', 'QR_STR_SCENE', 'QR_LIMIT_SCENE'] = 'QR_LIMIT_STR_SCENE',
                  hava_expire: bool = False, expire_seconds: int = 2592000):
    """
    创建二维码
    :param access_token:
    :param scene_value: 场景值ID
        - 整型参数值: 临时二维码时为32位非0整型，永久二维码时最大值为100000（目前参数只支持1--100000）
        - 字符串参数值: 字符串类型，长度限制为1到64
    :param action_name:
        - QR_SCENE为临时的整型参数值
        - QR_STR_SCENE为临时的字符串参数值
        - QR_LIMIT_SCENE为永久的整型参数值
        - QR_LIMIT_STR_SCENE为永久的字符串参数值
    :param hava_expire: 是否有过期时间
    :param expire_seconds: 二维码有效时间，以秒为单位。 最大不超过2592000（即30天），此字段如果不填，则默认有效期为60秒。
    :return:
    """
    params = dict()
    if hava_expire:
        params['expire_seconds'] = expire_seconds
    params['action_name'] = action_name
    if action_name == 'QR_SCENE' or action_name == 'QR_LIMIT_SCENE':
        params['action_info'] = {
            'scene_id': scene_value
        }
    else:
        params['action_info'] = {
            'scene_str': str(scene_value)
        }

    resp = requests.post(f'https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token={access_token}',
                         params={"expire_seconds": expire_seconds,
                                 "action_name": "QR_SCENE",
                                 "action_info": {"scene": {"scene_id": 123}}})
    resp.encoding = 'utf-8'
    json_resp = resp.json()
    print(resp.text)
    print()


if __name__ == '__main__':
    access_token = wx_token_tools.get_access_token(app_id=settings_tools.get_config('weixin.app_id'),
                                                   app_secret=settings_tools.get_config('weixin.app_secret'))

    print('access_token:', access_token)



from .. import aliyun_drive_auth_tools, aliyun_drive_share_tools


def run():
    alipan_api = aliyun_drive_auth_tools.get_alipan_api()
    share_token = aliyun_drive_share_tools.get_share_token(alipan_api, share_id='Y31kDxNjphx')
    share_file_list = aliyun_drive_share_tools.get_share_file_list(alipan_api, share_token=share_token,
                                                                   parent_file_id='64a9326505e0e8e41fb245ae842f726ccc914133')
    for tmp_share_file in share_file_list:
        tmp_share_info = aliyun_drive_share_tools.get_share_link_download_url(alipan_api,
                                                                              file_id=tmp_share_file.file_id,
                                                                              share_token=share_token)
        print()


def run_download_file_demo():
    alipan_api = aliyun_drive_auth_tools.get_alipan_api()
    share_token = aliyun_drive_share_tools.get_share_token(alipan_api, share_id='Y31kDxNjphx')
    share_file_list = aliyun_drive_share_tools.get_share_file_list(alipan_api, share_token=share_token,
                                                                   parent_file_id='64a9326505e0e8e41fb245ae842f726ccc914133')
    for tmp_share_file in share_file_list:
        tmp_file = aliyun_drive_share_tools.download_share_file(alipan_api, share_token=share_token,
                                                                share_file=tmp_share_file,
                                                                local_folder='./output')
        print(tmp_file)


def run_download_folder_demo():
    alipan_api = aliyun_drive_auth_tools.get_alipan_api()
    share_token = aliyun_drive_share_tools.get_share_token(alipan_api, share_id='Y31kDxNjphx')
    share_file = aliyun_drive_share_tools.get_share_file(alipan_api, share_token,
                                                         file_id='64a9326505e0e8e41fb245ae842f726ccc914133')
    result = aliyun_drive_share_tools.download_share_file(alipan_api, share_token=share_token,
                                                          share_file=share_file,
                                                          local_folder='./output')
    print(result)


if __name__ == '__main__':
    # run()
    # run_download_file_demo()
    run_download_folder_demo()



"""扩展实现彻底删除文件功能"""
from typing import List

from aligo import Aligo, BatchRequest, BatchSubRequest


class CustomAligo(Aligo):
    """自定义 aligo """
    V3_FILE_DELETE = '/v3/file/delete'

    def delete_file(self, file_id: str, drive_id: str = None) -> bool:
        """删除文件"""
        drive_id = drive_id or self.default_drive_id
        response = self.post(self.V3_FILE_DELETE, body={
            'drive_id': drive_id,
            'file_id': file_id
        })
        return response.status_code == 204

    def batch_delete_files(self, file_id_list: List[str], drive_id: str = None):
        """批量删除文件"""
        drive_id = drive_id or self.default_drive_id
        result = self.batch_request(BatchRequest(
            requests=[BatchSubRequest(
                id=file_id,
                url='/file/delete',
                body={
                    'drive_id': drive_id,
                    'file_id': file_id
                }
            ) for file_id in file_id_list]
        ), dict)
        return list(result)

    def clear_recyclebin(self, drive_id: str = None):
        """清空回收站"""
        drive_id = drive_id or self.default_drive_id
        response = self.post('/v2/recyclebin/clear', body={
            'drive_id': drive_id
        })
        return response.status_code == 202


if __name__ == '__main__':
    ali = CustomAligo()

    # x = ali.delete_file('646604d56d3ee7cfe2654b4as9ed4xse59cf76')
    # print(x)

    # 清空回收站
    # ll = ali.get_recyclebin_list()
    # rr = ali.batch_delete_files([f.file_id for f in ll])

    # 新的清空回收站方法，异步的
    rr = ali.clear_recyclebin()
    print(rr)


import re
import winreg


def get_desktop_path():
    """获取桌面路径"""
    _key = winreg.OpenKey(
        winreg.HKEY_CURRENT_USER,
        r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
    return winreg.QueryValueEx(_key, "Desktop")[0]


def del_special_symbol(s: str) -> str:
    """删除Windows文件名中不允许的字符"""
    return re.sub(r'[:*?"<>|]', '_', s)

print(get_desktop_path())


"""
认证工具
pip install aligo -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
from aligo import EMailConfig, set_config_folder

from .core.custom_aligo import CustomAligo


def get_alipan_api_by_email(receive_email: str, send_email: str, send_password: str, email_host: str,
                            email_port: int) -> CustomAligo:
    """
    获取api接口：发送登录二维码到邮箱(建议将邮箱绑定到微信，这样能实时收到提醒，登录过期后也可以第一时间收到登录请求。)
    :param receive_email:  接收登录邮件的邮箱地址
    :param send_email: 发送邮件的邮箱
    :param send_password: 发送邮件的密码
    :param email_host: 发送邮件的主机
    :param email_port: 发送邮件的端口
    :return: Aligo
    """
    email_config = EMailConfig(
        email=receive_email,
        # 自配邮箱
        user=send_email,
        password=send_password,
        host=email_host,
        port=email_port,
    )
    return CustomAligo(email=email_config)


def get_alipan_api_by_web(port: int, use_aria2: bool = False) -> CustomAligo:
    """获取api接口：打开浏览器访问 http://<YOUR_IP>:<port> 网页扫码登录"""
    return CustomAligo(port=port, use_aria2=use_aria2)


def get_alipan_api(name: str = None,
                   config_path: str = None,
                   use_resource_drive: bool = False,
                   use_aria2: bool = False,
                   email: EMailConfig = None,
                   port: int = None,
                   ) -> CustomAligo:
    """
    获取api接口：第一次使用，会弹出二维码，供扫描登录
    :param name: 配置文件名，如：name='一号服务器'， 会创建 <用户目录>/.alig/一号服务器.json 配置文件
    :param config_path: 配置文件目录，默认是 <用户目录>/.alig
    :param use_resource_drive: 是否是资源盘
    :param use_aria2: 是否使用 aria2 下载
    :param email:  (可选) 邮箱配置，发送登录二维码到邮箱, 参考 EMailConfig
    :param port: (可选) 开启 http server 端口，用于网页端扫码登录. 提供此值时，将不再弹出或打印二维码
    :return:
    """
    if config_path:
        set_config_folder(config_path)
    if name:
        alipan_api = CustomAligo(name=name, use_aria2=use_aria2, email=email)
    else:
        alipan_api = CustomAligo(use_aria2=use_aria2, email=email)
    if use_resource_drive:
        alipan_api.default_drive_id = alipan_api.v2_user_get().resource_drive_id
    return alipan_api
